@@ -1,7 +1,7 @@
 /* request.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: request.cc,v 1.31 2004/03/18 07:02:33 ralf Exp $ */
+/* $Id: request.cc,v 1.33 2004/10/31 15:56:20 ralf Exp $ */
 
 #include "request.h"
 #include "aguix.h"
@@ -29,6 +29,8 @@
 #include "stringgadget.h"
 #include "bevelbox.h"
 
+#define ACCEPT_ENTER_FOR_CHOOSE 1
+
 Requester::Requester(AGUIX *parent)
 {
   aguix=parent;
@@ -140,8 +142,7 @@ int Requester::request(const char *title,const char *text,const char *buttons, A
 	    switch(msg->key.key) {
 	      case XK_KP_Enter:
 	      case XK_Return:
-#if 0
-		endmode = 0;
+#ifdef ACCEPT_ENTER_FOR_CHOOSE
 		for ( i = 0; i < nbuttons; i++ ) {
 		  if ( buts[i]->hasFocus() == true ) {
 		    endmode = i;
@@ -198,6 +199,12 @@ int Requester::request(const char *title,const char *text,const char *buttons, A
 	        endmode=9;
 		if(endmode>=nbuttons) endmode = -1;
 	        break;
+	      case XK_Left:
+		win->prevFocus();
+		break;
+  	      case XK_Right:
+		win->nextFocus();
+		break;
 	    }
 	  }
 	  break;
@@ -340,8 +347,7 @@ int Requester::string_request( const char *title, const char *text, const char *
 	    switch(msg->key.key) {
 	      case XK_KP_Enter:
 	      case XK_Return:
-#if 0
-	        endmode=0;
+#ifdef ACCEPT_ENTER_FOR_CHOOSE
 		for ( i = 0; i < nbuttons; i++ ) {
 		  if ( buts[i]->hasFocus() == true ) {
 		    endmode = i;
@@ -398,6 +404,12 @@ int Requester::string_request( const char *title, const char *text, const char *
 	        endmode=9;
 		if(endmode>=nbuttons) endmode = -1;
 	        break;
+	      case XK_Left:
+		win->prevFocus();
+		break;
+  	      case XK_Right:
+		win->nextFocus();
+		break;
 	    }
 	  }
 	  break;
