@@ -1,7 +1,7 @@
 /* basic_actions.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: basic_actions.cc,v 1.19 2004/03/18 07:05:16 ralf Exp $ */
+/* $Id: basic_actions.cc,v 1.23 2004/12/12 22:00:23 ralf Exp $ */
 
 #include "basic_actions.h"
 #include "listermode.h"
@@ -83,25 +83,31 @@ void DNDAction::normalmodedndaction()
   Lister *l1 = am->getWorker()->getActiveLister();
   NormalMode *nm=NULL;
   ActionMessage amsg( am->getWorker() );
-  FileEntry *fe = NULL;
+  const FileEntry *fe = NULL;
+  NM_specialsourceExt *ss1;
 
   switch ( am->mode ) {
     case ActionMessage::AM_MODE_DNDACTION:
       break;
     case ActionMessage::AM_MODE_SPECIAL:
-      fe = (FileEntry*)am->extraInfo;
+      fe = am->getFE();
       if ( fe != NULL ) {
         WCFiletype *ft;
       
         amsg.mode = amsg.AM_MODE_SPECIAL;
       
-        ft = fe->filetype;
+	if ( am->filetype != NULL ) ft = am->filetype;
+	else ft = fe->filetype;
         if ( ft == NULL ) {
           ft = wconfig->getnotyettype(); // not yet checked
         }
         if ( ft != NULL ) {
-          amsg.extraInfo = fe;
+	  // FE is already a copy of the real FE so we don't need to make another copy
+	  // but setFE does make a copy currently which doesn't hurt
+          amsg.setFE( fe );
           amsg.flags = ft->getDNDActionsGF();
+	  amsg.filetype = ft;
+	  amsg.getActionList = DNDAction::getActionList;
           am->getWorker()->interpret( ft->getDNDActions(), &amsg );
         }
       }
@@ -114,12 +120,12 @@ void DNDAction::normalmodedndaction()
           if(lm1->isType("NormalMode")==true) nm=(NormalMode*)lm1;
       }
       if(nm!=NULL) {
-        List *files=new List();
+        std::list<NM_specialsourceExt*> *files=new std::list<NM_specialsourceExt*>;
+	std::list<NM_specialsourceExt*>::iterator it1;
         nm->getSelFiles( files,
                          ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? NormalMode::NM_GETFILES_ONLYACTIVE :
                                                                              NormalMode::NM_GETFILES_SELORACT,
                          false );
-        int id=files->initEnum();
         WCFiletype *ft;
         
         amsg.startLister=l1;
@@ -128,29 +134,34 @@ void DNDAction::normalmodedndaction()
         if(files->size()==0) {
           ft = wconfig->getvoidtype();
           if(ft!=NULL) {
-            amsg.extraInfo=fe;
+            amsg.setFE( NULL );
             amsg.flags=ft->getDNDActionsGF();
+	    amsg.filetype = ft;
+	    amsg.getActionList = DNDAction::getActionList;
             am->getWorker()->interpret(ft->getDNDActions(),&amsg);
           }
         } else {
-          NM_specialsource *ss1=(NM_specialsource*)files->getFirstElement(id);
-          while(ss1!=NULL) {
-            fe=ss1->entry;
+	  for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
+	    ss1 = *it1;
+            fe = ss1->entry();
             if(fe!=NULL) {
-              if(fe->isDir()==false) {
-                ft=fe->filetype;
-                if ( ft == NULL ) ft = wconfig->getnotyettype(); // not yet checked
-                if(ft!=NULL) {
-                  amsg.extraInfo=fe;
-                  amsg.flags=ft->getDNDActionsGF();
-                  am->getWorker()->interpret(ft->getDNDActions(),&amsg);
-                }
-              }
+	      ft=fe->filetype;
+	      if ( ft == NULL ) {
+		if ( fe->isDir() == true ) ft = wconfig->getdirtype();
+		else ft = wconfig->getnotyettype();
+	      }
+	      if(ft!=NULL) {
+		// NM_specialsourceExt already contains a duplicated FE so we can use
+		// it here without another copy
+		amsg.setFE( fe );
+		amsg.flags=ft->getDNDActionsGF();
+		amsg.filetype = ft;
+		amsg.getActionList = DNDAction::getActionList;
+		am->getWorker()->interpret(ft->getDNDActions(),&amsg);
+	      }
             }
-            _freesafe(ss1);
-            ss1=(NM_specialsource*)files->getNextElement(id);
+            delete ss1;
           }
-          files->closeEnum(id);
         }
         delete files;
       }
@@ -158,6 +169,17 @@ void DNDAction::normalmodedndaction()
   }
 }
 
+List *DNDAction::getActionList( class WCFiletype *filetype, int id, WCGlobalFlags **return_flags )
+{
+  if ( filetype == NULL ) return NULL;
+  
+  if ( return_flags != NULL ) {
+    *return_flags = filetype->getDNDActionsGF();
+  }
+
+  return filetype->getDNDActions();
+}
+
 /***************
  Ende DNDAction
 ****************/
@@ -219,46 +241,57 @@ void DoubleClickAction::normalmodeddaction()
   Lister *l1;
   NormalMode *nm=NULL;
   ActionMessage amsg( am->getWorker() );
-  FileEntry *fe = NULL;
+  const FileEntry *fe = NULL;
+  NM_specialsourceExt *ss1;
 
   switch ( am->mode ) {
     case ActionMessage::AM_MODE_DNDACTION:
-      l1=((DNDMsg*)am->extraInfo)->getSourceLister();
+      l1 = am->dndmsg->getSourceLister();
       if(l1==NULL) return;
   
-      fe = ((DNDMsg*)am->extraInfo)->getFE();
+      fe = am->dndmsg->getFE();
       if(fe!=NULL) {
         WCFiletype *ft;
       
         amsg.startLister=l1;
         amsg.mode=amsg.AM_MODE_SPECIAL;
       
-        ft=fe->filetype;
+	if ( am->filetype != NULL ) ft = am->filetype;
+	else ft = fe->filetype;
         if(ft==NULL) {
           if ( fe->isDir() == false ) ft = wconfig->getnotyettype(); // not yet checked
           else ft = wconfig->getdirtype(); // dir
         }
         if(ft!=NULL) {
-          amsg.extraInfo=fe;
+	  // I need to make a copy of fe as DNDMsg->getFE returns a pointer to the actual FE
+	  // setFE takes care of this at the moment
+          amsg.setFE( fe );
           amsg.flags=ft->getDoubleClickActionsGF();
+	  amsg.filetype = ft;
+	  amsg.getActionList = DoubleClickAction::getActionList;
           am->getWorker()->interpret(ft->getDoubleClickActions(),&amsg);
         }
       }
       break;
     case ActionMessage::AM_MODE_SPECIAL:
-      fe = (FileEntry*)am->extraInfo;
+      fe = am->getFE();
       if ( fe != NULL ) {
         WCFiletype *ft;
       
         amsg.mode = amsg.AM_MODE_SPECIAL;
       
-        ft = fe->filetype;
+	if ( am->filetype != NULL ) ft = am->filetype;
+	else ft = fe->filetype;
         if ( ft == NULL ) {
           ft = wconfig->getnotyettype(); // not yet checked
         }
         if ( ft != NULL ) {
-          amsg.extraInfo = fe;
+	  // FE is already a copy of the real FE so we don't need to make another copy
+	  // but setFE does make a copy currently which doesn't hurt
+          amsg.setFE( fe );
           amsg.flags = ft->getDoubleClickActionsGF();
+	  amsg.filetype = ft;
+	  amsg.getActionList = DoubleClickAction::getActionList;
           am->getWorker()->interpret( ft->getDoubleClickActions(), &amsg );
         }
       }
@@ -272,12 +305,12 @@ void DoubleClickAction::normalmodeddaction()
           if(lm1->isType("NormalMode")==true) nm=(NormalMode*)lm1;
       }
       if(nm!=NULL) {
-        List *files=new List();
+        std::list<NM_specialsourceExt*> *files = new std::list<NM_specialsourceExt*>;
+	std::list<NM_specialsourceExt*>::iterator it1;
         nm->getSelFiles( files,
                          ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? NormalMode::NM_GETFILES_ONLYACTIVE :
                                                                              NormalMode::NM_GETFILES_SELORACT,
                          false );
-        int id=files->initEnum();
         WCFiletype *ft;
       
         amsg.startLister=l1;
@@ -286,29 +319,34 @@ void DoubleClickAction::normalmodeddaction()
         if(files->size()==0) {
           ft = wconfig->getvoidtype();
           if(ft!=NULL) {
-            amsg.extraInfo=fe;
+            amsg.setFE( NULL );
             amsg.flags=ft->getDoubleClickActionsGF();
+	    amsg.filetype = ft;
+	    amsg.getActionList = DoubleClickAction::getActionList;
             am->getWorker()->interpret(ft->getDoubleClickActions(),&amsg);
           }
         } else {
-          NM_specialsource *ss1=(NM_specialsource*)files->getFirstElement(id);
-          while(ss1!=NULL) {
-            fe=ss1->entry;
+	  for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
+	    ss1 = *it1;
+            fe = ss1->entry();
             if(fe!=NULL) {
-              if(fe->isDir()==false) {
-                ft=fe->filetype;
-                if ( ft == NULL ) ft = wconfig->getnotyettype(); // not yet checked
-                if(ft!=NULL) {
-                  amsg.extraInfo=fe;
-                  amsg.flags=ft->getDoubleClickActionsGF();
-                  am->getWorker()->interpret(ft->getDoubleClickActions(),&amsg);
-                }
-              }
+	      ft=fe->filetype;
+	      if ( ft == NULL ) {
+		if ( fe->isDir() == true ) ft = wconfig->getdirtype();
+		else ft = wconfig->getnotyettype();
+	      }
+	      if(ft!=NULL) {
+		// NM_specialsourceExt already contains a duplicated FE so we can use
+		// it here without another copy
+		amsg.setFE( fe );
+		amsg.flags=ft->getDoubleClickActionsGF();
+		amsg.filetype = ft;
+		amsg.getActionList = DoubleClickAction::getActionList;
+		am->getWorker()->interpret(ft->getDoubleClickActions(),&amsg);
+	      }
             }
-            _freesafe(ss1);
-            ss1=(NM_specialsource*)files->getNextElement(id);
+            delete ss1;
           }
-          files->closeEnum(id);
         }
         delete files;
       }
@@ -316,6 +354,17 @@ void DoubleClickAction::normalmodeddaction()
   }
 }
 
+List *DoubleClickAction::getActionList( class WCFiletype *filetype, int id, WCGlobalFlags **return_flags )
+{
+  if ( filetype == NULL ) return NULL;
+  
+  if ( return_flags != NULL ) {
+    *return_flags = filetype->getDoubleClickActionsGF();
+  }
+
+  return filetype->getDoubleClickActions();
+}
+
 /***************
  Ende DoubleClickAction
 ****************/
@@ -377,46 +426,57 @@ void ShowAction::normalmodeshow()
   NormalMode *nm=NULL;
   ActionMessage amsg( am->getWorker() );
   Lister *l1;
-  FileEntry *fe = NULL;
+  const FileEntry *fe = NULL;
+  NM_specialsourceExt *ss1;
 
   switch ( am->mode ) {
     case ActionMessage::AM_MODE_DNDACTION:
-      l1=((DNDMsg*)am->extraInfo)->getSourceLister();
+      l1 = am->dndmsg->getSourceLister();
       if(l1==NULL) return;
   
-      fe=((DNDMsg*)am->extraInfo)->getFE();
+      fe = am->dndmsg->getFE();
       if(fe!=NULL) {
         WCFiletype *ft;
       
         amsg.startLister=l1;
         amsg.mode=amsg.AM_MODE_SPECIAL;
       
-        ft=fe->filetype;
+	if ( am->filetype != NULL ) ft = am->filetype;
+	else ft = fe->filetype;
         if(ft==NULL) {
           if ( fe->isDir() == false ) ft = wconfig->getnotyettype(); // not yet checked
           else ft = wconfig->getdirtype(); // dir
         }
         if(ft!=NULL) {
-          amsg.extraInfo=fe;
+	  // I need to make a copy of fe as DNDMsg->getFE returns a pointer to the actual FE
+	  // setFE takes care of this at the moment
+          amsg.setFE( fe );
           amsg.flags=ft->getShowActionsGF();
+	  amsg.filetype = ft;
+	  amsg.getActionList = ShowAction::getActionList;
           am->getWorker()->interpret(ft->getShowActions(),&amsg);
         }
       }
       break;
     case ActionMessage::AM_MODE_SPECIAL:
-      fe = (FileEntry*)am->extraInfo;
+      fe = am->getFE();
       if ( fe != NULL ) {
         WCFiletype *ft;
       
         amsg.mode = amsg.AM_MODE_SPECIAL;
       
-        ft = fe->filetype;
+	if ( am->filetype != NULL ) ft = am->filetype;
+	else ft = fe->filetype;
         if ( ft == NULL ) {
           ft = wconfig->getnotyettype(); // not yet checked
         }
         if ( ft != NULL ) {
-          amsg.extraInfo = fe;
+	  // FE is already a copy of the real FE so we don't need to make another copy
+	  // but setFE does make a copy currently which doesn't hurt
+          amsg.setFE( fe );
           amsg.flags = ft->getShowActionsGF();
+	  amsg.filetype = ft;
+	  amsg.getActionList = ShowAction::getActionList;
           am->getWorker()->interpret( ft->getShowActions(), &amsg );
         }
       }
@@ -430,12 +490,12 @@ void ShowAction::normalmodeshow()
           if(lm1->isType("NormalMode")==true) nm=(NormalMode*)lm1;
       }
       if(nm!=NULL) {
-        List *files=new List();
+        std::list<NM_specialsourceExt*> *files = new std::list<NM_specialsourceExt*>;
+	std::list<NM_specialsourceExt*>::iterator it1;
         nm->getSelFiles( files,
                          ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? NormalMode::NM_GETFILES_ONLYACTIVE :
                                                                              NormalMode::NM_GETFILES_SELORACT,
                          false );
-        int id=files->initEnum();
         WCFiletype *ft;
       
         amsg.startLister=l1;
@@ -444,29 +504,34 @@ void ShowAction::normalmodeshow()
         if(files->size()==0) {
           ft = wconfig->getvoidtype();
           if(ft!=NULL) {
-            amsg.extraInfo=fe;
+            amsg.setFE( NULL );
             amsg.flags=ft->getShowActionsGF();
+	    amsg.filetype = ft;
+	    amsg.getActionList = ShowAction::getActionList;
             am->getWorker()->interpret(ft->getShowActions(),&amsg);
           }
         } else {
-          NM_specialsource *ss1=(NM_specialsource*)files->getFirstElement(id);
-          while(ss1!=NULL) {
-            fe=ss1->entry;
+	  for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
+	    ss1 = *it1;
+            fe = ss1->entry();
             if(fe!=NULL) {
-              if(fe->isDir()==false) {
-                ft=fe->filetype;
-                if ( ft == NULL ) ft = wconfig->getnotyettype(); // not yet checked
-                if(ft!=NULL) {
-                  amsg.extraInfo=fe;
-                  amsg.flags=ft->getShowActionsGF();
-                  am->getWorker()->interpret(ft->getShowActions(),&amsg);
-                }
-              }
+	      ft=fe->filetype;
+	      if ( ft == NULL ) {
+		if ( fe->isDir() == true ) ft = wconfig->getdirtype();
+		else ft = wconfig->getnotyettype();
+	      }
+	      if(ft!=NULL) {
+		// NM_specialsourceExt already contains a duplicated FE so we can use
+		// it here without another copy
+		amsg.setFE( fe );
+		amsg.flags=ft->getShowActionsGF();
+		amsg.filetype = ft;
+		amsg.getActionList = ShowAction::getActionList;
+		am->getWorker()->interpret(ft->getShowActions(),&amsg);
+	      }
             }
-            _freesafe(ss1);
-            ss1=(NM_specialsource*)files->getNextElement(id);
+            delete ss1;
           }
-          files->closeEnum(id);
         }
         delete files;
       }
@@ -474,6 +539,17 @@ void ShowAction::normalmodeshow()
   }
 }
 
+List *ShowAction::getActionList( class WCFiletype *filetype, int id, WCGlobalFlags **return_flags )
+{
+  if ( filetype == NULL ) return NULL;
+  
+  if ( return_flags != NULL ) {
+    *return_flags = filetype->getShowActionsGF();
+  }
+
+  return filetype->getShowActions();
+}
+
 /***************
  Ende ShowAction
 ****************/
@@ -535,46 +611,57 @@ void RawShowAction::normalmoderawshow()
   NormalMode *nm=NULL;
   ActionMessage amsg( am->getWorker() );
   Lister *l1;
-  FileEntry *fe = NULL;
+  const FileEntry *fe = NULL;
+  NM_specialsourceExt *ss1;
 
   switch ( am->mode ) {
     case ActionMessage::AM_MODE_DNDACTION:
-      l1=((DNDMsg*)am->extraInfo)->getSourceLister();
+      l1 = am->dndmsg->getSourceLister();
       if(l1==NULL) return;
   
-      fe=((DNDMsg*)am->extraInfo)->getFE();
+      fe = am->dndmsg->getFE();
       if(fe!=NULL) {
         WCFiletype *ft;
       
         amsg.startLister=l1;
         amsg.mode=amsg.AM_MODE_SPECIAL;
       
-        ft=fe->filetype;
+	if ( am->filetype != NULL ) ft = am->filetype;
+	else ft = fe->filetype;
         if(ft==NULL) {
           if ( fe->isDir() == false ) ft = wconfig->getnotyettype(); // not yet checked
           else ft = wconfig->getdirtype(); // dir
         }
         if(ft!=NULL) {
-          amsg.extraInfo=fe;
+	  // I need to make a copy of fe as DNDMsg->getFE returns a pointer to the actual FE
+	  // setFE takes care of this at the moment
+          amsg.setFE( fe );
           amsg.flags=ft->getRawShowActionsGF();
+	  amsg.filetype = ft;
+	  amsg.getActionList = RawShowAction::getActionList;
           am->getWorker()->interpret(ft->getRawShowActions(),&amsg);
         }
       }
       break;
     case ActionMessage::AM_MODE_SPECIAL:
-      fe = (FileEntry*)am->extraInfo;
+      fe = am->getFE();
       if ( fe != NULL ) {
         WCFiletype *ft;
       
         amsg.mode = amsg.AM_MODE_SPECIAL;
       
-        ft = fe->filetype;
+	if ( am->filetype != NULL ) ft = am->filetype;
+	else ft = fe->filetype;
         if ( ft == NULL ) {
           ft = wconfig->getnotyettype(); // not yet checked
         }
         if ( ft != NULL ) {
-          amsg.extraInfo = fe;
+	  // FE is already a copy of the real FE so we don't need to make another copy
+	  // but setFE does make a copy currently which doesn't hurt
+          amsg.setFE( fe );
           amsg.flags = ft->getRawShowActionsGF();
+	  amsg.filetype = ft;
+	  amsg.getActionList = RawShowAction::getActionList;
           am->getWorker()->interpret( ft->getRawShowActions(), &amsg );
         }
       }
@@ -588,12 +675,12 @@ void RawShowAction::normalmoderawshow()
           if(lm1->isType("NormalMode")==true) nm=(NormalMode*)lm1;
       }
       if(nm!=NULL) {
-        List *files=new List();
+        std::list<NM_specialsourceExt*> *files = new std::list<NM_specialsourceExt*>;
+	std::list<NM_specialsourceExt*>::iterator it1;
         nm->getSelFiles( files,
                          ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? NormalMode::NM_GETFILES_ONLYACTIVE :
                                                                              NormalMode::NM_GETFILES_SELORACT,
                          false );
-        int id=files->initEnum();
         WCFiletype *ft;
       
         amsg.startLister=l1;
@@ -602,29 +689,34 @@ void RawShowAction::normalmoderawshow()
         if(files->size()==0) {
           ft = wconfig->getvoidtype();
           if(ft!=NULL) {
-            amsg.extraInfo=fe;
+            amsg.setFE( NULL );
             amsg.flags=ft->getRawShowActionsGF();
+	    amsg.filetype = ft;
+	    amsg.getActionList = RawShowAction::getActionList;
             am->getWorker()->interpret(ft->getRawShowActions(),&amsg);
           }
         } else {
-          NM_specialsource *ss1=(NM_specialsource*)files->getFirstElement(id);
-          while(ss1!=NULL) {
-            fe=ss1->entry;
+	  for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
+	    ss1 = *it1;
+            fe = ss1->entry();
             if(fe!=NULL) {
-              if(fe->isDir()==false) {
-                ft=fe->filetype;
-                if ( ft == NULL ) ft = wconfig->getnotyettype(); // not yet checked
-                if(ft!=NULL) {
-                  amsg.extraInfo=fe;
-                  amsg.flags=ft->getRawShowActionsGF();
-                  am->getWorker()->interpret(ft->getRawShowActions(),&amsg);
-                }
-              }
+	      ft=fe->filetype;
+	      if ( ft == NULL ) {
+		if ( fe->isDir() == true ) ft = wconfig->getdirtype();
+		else ft = wconfig->getnotyettype();
+	      }
+	      if(ft!=NULL) {
+		// NM_specialsourceExt already contains a duplicated FE so we can use
+		// it here without another copy
+		amsg.setFE( fe );
+		amsg.flags=ft->getRawShowActionsGF();
+		amsg.filetype = ft;
+		amsg.getActionList = RawShowAction::getActionList;
+		am->getWorker()->interpret(ft->getRawShowActions(),&amsg);
+	      }
             }
-            _freesafe(ss1);
-            ss1=(NM_specialsource*)files->getNextElement(id);
+            delete ss1;
           }
-          files->closeEnum(id);
         }
         delete files;
       }
@@ -632,6 +724,17 @@ void RawShowAction::normalmoderawshow()
   }
 }
 
+List *RawShowAction::getActionList( class WCFiletype *filetype, int id, WCGlobalFlags **return_flags )
+{
+  if ( filetype == NULL ) return NULL;
+  
+  if ( return_flags != NULL ) {
+    *return_flags = filetype->getRawShowActionsGF();
+  }
+
+  return filetype->getRawShowActions();
+}
+
 /***************
  Ende RawShowAction
 ****************/
@@ -696,46 +799,59 @@ void UserAction::normalmodeuseraction()
   NormalMode *nm=NULL;
   ActionMessage amsg( am->getWorker() );
   Lister *l1;
-  FileEntry *fe = NULL;
+  const FileEntry *fe = NULL;
+  NM_specialsourceExt *ss1;
 
   switch ( am->mode ) {
     case ActionMessage::AM_MODE_DNDACTION:
-      l1=((DNDMsg*)am->extraInfo)->getSourceLister();
+      l1 = am->dndmsg->getSourceLister();
       if(l1==NULL) return;
   
-      fe=((DNDMsg*)am->extraInfo)->getFE();
+      fe = am->dndmsg->getFE();
       if(fe!=NULL) {
         WCFiletype *ft;
       
         amsg.startLister=l1;
         amsg.mode=amsg.AM_MODE_SPECIAL;
       
-        ft=fe->filetype;
+	if ( am->filetype != NULL ) ft = am->filetype;
+	else ft = fe->filetype;
         if(ft==NULL) {
           if ( fe->isDir() == false ) ft = wconfig->getnotyettype(); // not yet checked
           else ft = wconfig->getdirtype(); // dir
         }
         if(ft!=NULL) {
-          amsg.extraInfo=fe;
+	  // I need to make a copy of fe as DNDMsg->getFE returns a pointer to the actual FE
+	  // setFE takes care of this at the moment
+          amsg.setFE( fe );
           amsg.flags=ft->getUserActionsGF(nr);
+	  amsg.filetype = ft;
+	  amsg.getActionList = UserAction::getActionList;
+	  amsg.getActionListID = nr;
           am->getWorker()->interpret(ft->getUserActions(nr),&amsg);
         }
       }
       break;
     case ActionMessage::AM_MODE_SPECIAL:
-      fe = (FileEntry*)am->extraInfo;
+      fe = am->getFE();
       if ( fe != NULL ) {
         WCFiletype *ft;
       
         amsg.mode = amsg.AM_MODE_SPECIAL;
       
-        ft = fe->filetype;
+	if ( am->filetype != NULL ) ft = am->filetype;
+	else ft = fe->filetype;
         if ( ft == NULL ) {
           ft = wconfig->getnotyettype(); // not yet checked
         }
         if ( ft != NULL ) {
-          amsg.extraInfo = fe;
+	  // FE is already a copy of the real FE so we don't need to make another copy
+	  // but setFE does make a copy currently which doesn't hurt
+          amsg.setFE( fe );
           amsg.flags = ft->getUserActionsGF( nr );
+	  amsg.filetype = ft;
+	  amsg.getActionList = UserAction::getActionList;
+	  amsg.getActionListID = nr;
           am->getWorker()->interpret( ft->getUserActions( nr ), &amsg );
         }
       }
@@ -749,12 +865,12 @@ void UserAction::normalmodeuseraction()
           if(lm1->isType("NormalMode")==true) nm=(NormalMode*)lm1;
       }
       if(nm!=NULL) {
-        List *files=new List();
+        std::list<NM_specialsourceExt*> *files = new std::list<NM_specialsourceExt*>;
+	std::list<NM_specialsourceExt*>::iterator it1;
         nm->getSelFiles( files,
                          ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? NormalMode::NM_GETFILES_ONLYACTIVE :
                                                                              NormalMode::NM_GETFILES_SELORACT,
                          false );
-        int id=files->initEnum();
         WCFiletype *ft;
       
         amsg.startLister=l1;
@@ -763,29 +879,36 @@ void UserAction::normalmodeuseraction()
         if(files->size()==0) {
           ft = wconfig->getvoidtype();
           if(ft!=NULL) {
-            amsg.extraInfo=fe;
+            amsg.setFE( NULL );
             amsg.flags=ft->getUserActionsGF(nr);
+	    amsg.filetype = ft;
+	    amsg.getActionList = UserAction::getActionList;
+	    amsg.getActionListID = nr;
             am->getWorker()->interpret(ft->getUserActions(nr),&amsg);
           }
         } else {
-          NM_specialsource *ss1=(NM_specialsource*)files->getFirstElement(id);
-          while(ss1!=NULL) {
-            fe=ss1->entry;
+	  for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
+	    ss1 = *it1;
+            fe = ss1->entry();
             if(fe!=NULL) {
-              if(fe->isDir()==false) {
-                ft=fe->filetype;
-                if ( ft == NULL ) ft = wconfig->getnotyettype(); // not yet checked
-                if(ft!=NULL) {
-                  amsg.extraInfo=fe;
-                  amsg.flags=ft->getUserActionsGF(nr);
-                  am->getWorker()->interpret(ft->getUserActions(nr),&amsg);
-                }
-              }
+	      ft=fe->filetype;
+	      if ( ft == NULL ) {
+		if ( fe->isDir() == true ) ft = wconfig->getdirtype();
+		else ft = wconfig->getnotyettype();
+	      }
+	      if(ft!=NULL) {
+		// NM_specialsourceExt already contains a duplicated FE so we can use
+		// it here without another copy
+		amsg.setFE( fe );
+		amsg.flags=ft->getUserActionsGF(nr);
+		amsg.filetype = ft;
+		amsg.getActionList = UserAction::getActionList;
+		amsg.getActionListID = nr;
+		am->getWorker()->interpret(ft->getUserActions(nr),&amsg);
+	      }
             }
-            _freesafe(ss1);
-            ss1=(NM_specialsource*)files->getNextElement(id);
+            delete ss1;
           }
-          files->closeEnum(id);
         }
         delete files;
       }
@@ -909,6 +1032,17 @@ void UserAction::setNr( int n )
   if ( nr > 9 ) nr = 9;
 }
 
+List *UserAction::getActionList( class WCFiletype *filetype, int id, WCGlobalFlags **return_flags )
+{
+  if ( filetype == NULL ) return NULL;
+  
+  if ( return_flags != NULL ) {
+    *return_flags = filetype->getUserActionsGF( id );
+  }
+
+  return filetype->getUserActions( id );
+}
+
 /***************
  Ende UserAction
 ****************/
