@@ -1,7 +1,7 @@
 /* condparser.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: condparser.cc,v 1.3 2004/07/25 19:51:12 ralf Exp $ */
+/* $Id: condparser.cc,v 1.6 2004/10/27 21:06:21 ralf Exp $ */
 
 #include "condparser.h"
 #include <aguix/lowlevelfunc.h>
@@ -79,6 +79,7 @@ CondParser::CondParser()
   scanbuflen = -1;
   lasterror = 0;
   fe = NULL;
+  ignoreContent = false;
 }
 
 CondParser::~CondParser()
@@ -771,11 +772,15 @@ CondParser::erg_t *CondParser::factor()
         e->strval = (char*)_allocsafe( A_BYTESFORNUMBER( e->numval ) );
         sprintf( e->strval, "%d", e->numval );
       } else {
-        if ( e2 != NULL ) {
-          tstr = fe->getContentStr( e->numval, e2->numval );
-        } else {
-          tstr = fe->getContentStr( e->numval );
-        }
+	if ( ignoreContent == false ) {
+	  if ( e2 != NULL ) {
+	    tstr = fe->getContentStr( e->numval, e2->numval );
+	  } else {
+	    tstr = fe->getContentStr( e->numval );
+	  }
+	} else {
+	  tstr = dupstring( "" );
+	}
         if ( tstr != NULL ) {
           _freesafe( e->strval );
           e->strval = tstr;
@@ -818,11 +823,15 @@ CondParser::erg_t *CondParser::factor()
         e->strval = (char*)_allocsafe( A_BYTESFORNUMBER( e->numval ) );
         sprintf( e->strval, "%d", e->numval );
       } else {
-        if ( e2 != NULL ) {
-          e->numval = fe->getContentNum( e->numval, e2->numval );
-        } else {
-          e->numval = fe->getContentNum( e->numval );
-        }
+	if ( ignoreContent == false ) {
+	  if ( e2 != NULL ) {
+	    e->numval = fe->getContentNum( e->numval, e2->numval );
+	  } else {
+	    e->numval = fe->getContentNum( e->numval );
+	  }
+	} else {
+	  e->numval = 0;
+	}
         e->wasstring = 0;
         _freesafe( e->strval );
         e->strval = (char*)_allocsafe( A_BYTESFORNUMBER( e->numval ) );
@@ -1148,3 +1157,43 @@ const char *CondParser::requestFlag()
 
   return returnstr;
 }
+
+char *CondParser::getOutput( const char *str, FileEntry *tfe )
+{
+  if ( ( str == NULL ) || ( tfe == NULL ) ) return NULL;
+  char *erg = NULL;
+  int retval = 0;
+  int exeerror = 0;
+  ExeClass *ec;
+  char *tstr, *tstr2;
+
+  fe = tfe;
+  tstr = replaceFlags( str );
+  
+  if ( tstr != NULL ) {
+    ec = new ExeClass( ExeClass::EXECUTE_NORMAL );
+    tstr2 = ParentDir( fe->fullname, NULL );
+    if ( tstr2 != NULL ) {
+      ec->cdDir( tstr2 );
+      _freesafe( tstr2 );
+    }
+    ec->addCommand( "%s", tstr );
+    if ( ec->getOutputAndRV( &erg, &retval, &exeerror ) != 0 ) {
+      erg = NULL;
+    }
+    delete ec;
+    _freesafe( tstr );
+  }
+  fe = NULL;
+  return erg;
+}
+
+void CondParser::setIgnoreContent( bool nv )
+{
+  ignoreContent = nv;
+}
+
+bool CondParser::getIgnoreContent() const
+{
+  return ignoreContent;
+}
