@@ -285,12 +285,24 @@
      LISTVIEWSVERT_WCP = 508,
      EXTCOND_WCP = 509,
      USEEXTCOND_WCP = 510,
-     ERROR = 511,
-     ACTIVESIDE_WCP = 512,
-     SHOWFREESPACE_WCP = 513,
-     ACTIVEMODE_WCP = 514,
-     STRING_WCP = 515,
-     NUM_WCP = 516
+     SUBTYPE_WCP = 511,
+     PARENTACTIONOP_WCP = 512,
+     NOOPERATIONOP_WCP = 513,
+     COLORMODE_WCP = 514,
+     DEFAULT_WCP = 515,
+     CUSTOM_WCP = 516,
+     UNSELECTCOLOR_WCP = 517,
+     SELECTCOLOR_WCP = 518,
+     UNSELECTACTIVECOLOR_WCP = 519,
+     SELECTACTIVECOLOR_WCP = 520,
+     COLOREXTERNPROG_WCP = 521,
+     PARENT_WCP = 522,
+     ERROR = 523,
+     ACTIVESIDE_WCP = 524,
+     SHOWFREESPACE_WCP = 525,
+     ACTIVEMODE_WCP = 526,
+     STRING_WCP = 527,
+     NUM_WCP = 528
    };
 #endif
 #define LEFTBRACE_WCP 258
@@ -546,24 +558,36 @@
 #define LISTVIEWSVERT_WCP 508
 #define EXTCOND_WCP 509
 #define USEEXTCOND_WCP 510
-#define ERROR 511
-#define ACTIVESIDE_WCP 512
-#define SHOWFREESPACE_WCP 513
-#define ACTIVEMODE_WCP 514
-#define STRING_WCP 515
-#define NUM_WCP 516
+#define SUBTYPE_WCP 511
+#define PARENTACTIONOP_WCP 512
+#define NOOPERATIONOP_WCP 513
+#define COLORMODE_WCP 514
+#define DEFAULT_WCP 515
+#define CUSTOM_WCP 516
+#define UNSELECTCOLOR_WCP 517
+#define SELECTCOLOR_WCP 518
+#define UNSELECTACTIVECOLOR_WCP 519
+#define SELECTACTIVECOLOR_WCP 520
+#define COLOREXTERNPROG_WCP 521
+#define PARENT_WCP 522
+#define ERROR 523
+#define ACTIVESIDE_WCP 524
+#define SHOWFREESPACE_WCP 525
+#define ACTIVEMODE_WCP 526
+#define STRING_WCP 527
+#define NUM_WCP 528
 
 
 
 
 #ifndef YYSTYPE
-#line 190 "configparser.y"
+#line 192 "configparser.y"
 typedef union {
   int num;
   char *strptr;
 } yystype;
 /* Line 1281 of /usr/share/bison/yacc.c.  */
-#line 567 "configparser.tab.h"
+#line 591 "configparser.tab.h"
 # define YYSTYPE yystype
 #endif
 
