@@ -1,7 +1,7 @@
 /* configtokens.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2003-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: configtokens.cc,v 1.12 2004/06/13 21:21:28 ralf Exp $ */
+/* $Id: configtokens.cc,v 1.15 2004/10/27 21:06:34 ralf Exp $ */
 
 #include "wdefines.h"
 #include <aguix/util.h>
@@ -279,7 +279,19 @@ struct mytoken tokenlist[]={
   { BUTTONSVERT_WCP, "buttonsvertical" },
   { LISTVIEWSVERT_WCP, "listviewsvertical" },
   { EXTCOND_WCP, "extcond" },
-  { USEEXTCOND_WCP, "useextcond" }
+  { USEEXTCOND_WCP, "useextcond" },
+  { SUBTYPE_WCP, "subtype" },
+  { PARENTACTIONOP_WCP, "parentactionop" },
+  { NOOPERATIONOP_WCP, "nooperationop" },
+  { COLORMODE_WCP, "colormode" },
+  { CUSTOM_WCP, "custom" },
+  { DEFAULT_WCP, "default" },
+  { COLOREXTERNPROG_WCP, "colorexternprog" },
+  { UNSELECTCOLOR_WCP, "unselectcolor" },
+  { SELECTCOLOR_WCP, "selectcolor" },
+  { UNSELECTACTIVECOLOR_WCP, "unselectactivecolor" },
+  { SELECTACTIVECOLOR_WCP, "selectactivecolor" },
+  { PARENT_WCP, "parent" }
 };
 
 ArrayList *tokenlist_sorted = NULL;
