@@ -1,7 +1,7 @@
 /* copyop.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: copyop.cc,v 1.62 2004/07/01 18:55:46 ralf Exp $ */
+/* $Id: copyop.cc,v 1.64 2004/12/12 21:59:12 ralf Exp $ */
 
 #include "listermode.h"
 #include "normalmode.h"
@@ -143,16 +143,16 @@ CopyOp::normalmodecopy()
   NormalMode *nm1=NULL,
              *nm2=NULL;
   ListerMode *lm1=NULL;
-  struct NM_specialsource *specialsource=NULL;
+  NM_specialsourceExt *specialsource=NULL;
   bool do_request,cont=true;
   char *tstr,*tstr2,*dest=NULL;
   
   if(am->mode==am->AM_MODE_DNDACTION) {
-    lm1=((DNDMsg*)am->extraInfo)->getSourceMode();
+    lm1 = am->dndmsg->getSourceMode();
     if(lm1==NULL) return 1;
     if(lm1->isType("NormalMode")!=true) return 1;
     nm1=(NormalMode*)lm1;
-    lm1=((DNDMsg*)am->extraInfo)->getDestMode();
+    lm1 = am->dndmsg->getDestMode();
     if(lm1!=NULL)
       if(lm1->isType("NormalMode")==true)
         nm2=(NormalMode*)lm1;
@@ -193,18 +193,16 @@ CopyOp::normalmodecopy()
     else if(am->mode==am->AM_MODE_DNDACTION) {
       // insert DND-element into list
       copyorder.source=copyorder.NM_SPECIAL;
-      copyorder.sources=new List;
-      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
-      specialsource->entry=((DNDMsg*)am->extraInfo)->getFE();
+      copyorder.sources=new std::list<NM_specialsourceExt*>;
+      specialsource = new NM_specialsourceExt( am->dndmsg->getFE() );
       specialsource->row=-1;
-      copyorder.sources->addElement(specialsource);
+      copyorder.sources->push_back(specialsource);
     } else if(am->mode==am->AM_MODE_SPECIAL) {
       copyorder.source=copyorder.NM_SPECIAL;
-      copyorder.sources=new List;
-      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
-      specialsource->entry=(FileEntry*)am->extraInfo;
+      copyorder.sources=new std::list<NM_specialsourceExt*>;
+      specialsource = new NM_specialsourceExt( am->getFE() );
       specialsource->row=-1;
-      copyorder.sources->addElement(specialsource);
+      copyorder.sources->push_back(specialsource);
     } else
       copyorder.source=copyorder.NM_ALLENTRIES;
 
@@ -228,7 +226,7 @@ CopyOp::normalmodecopy()
       if(tstr!=NULL)
         dest=dupstring(tstr);
     } else if(am->mode==am->AM_MODE_DNDACTION) {
-      const char *ddstr=((DNDMsg*)am->extraInfo)->getDestDir();
+      const char *ddstr = am->dndmsg->getDestDir();
       if(ddstr!=NULL) dest=dupstring(ddstr);
       else dest=NULL;
     } else {
@@ -308,7 +306,7 @@ CopyOp::normalmodecopy()
     }
     if(dest!=NULL) _freesafe(dest);
     if(copyorder.source==copyorder.NM_SPECIAL) {
-      if(specialsource!=NULL) _freesafe(specialsource);
+      if ( specialsource != NULL ) delete specialsource;
       delete copyorder.sources;
     }
   }
