@@ -1,7 +1,7 @@
 /* createsymlinkop.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: createsymlinkop.cc,v 1.23 2004/03/18 07:06:22 ralf Exp $ */
+/* $Id: createsymlinkop.cc,v 1.25 2004/12/12 21:58:58 ralf Exp $ */
 
 #include "createsymlinkop.h"
 #include "listermode.h"
@@ -102,18 +102,18 @@ CreateSymlinkOp::normalmodecreatesl()
   NormalMode *nm1=NULL,
              *nm2=NULL;
   ListerMode *lm1=NULL;
-  struct NM_specialsource *specialsource=NULL;
+  NM_specialsourceExt *specialsource=NULL;
   bool cont=true;
   char *tstr,*dest=NULL;
   struct NM_createsymlinkorder cslorder;
   bool takesamedir;
   
   if(am->mode==am->AM_MODE_DNDACTION) {
-    lm1=((DNDMsg*)am->extraInfo)->getSourceMode();
+    lm1 = am->dndmsg->getSourceMode();
     if(lm1==NULL) return 1;
     if(lm1->isType("NormalMode")!=true) return 1;
     nm1=(NormalMode*)lm1;
-    lm1=((DNDMsg*)am->extraInfo)->getDestMode();
+    lm1 = am->dndmsg->getDestMode();
     if(lm1!=NULL)
       if(lm1->isType("NormalMode")==true)
         nm2=(NormalMode*)lm1;
@@ -147,24 +147,22 @@ CreateSymlinkOp::normalmodecreatesl()
     else if(am->mode==am->AM_MODE_DNDACTION) {
       // insert DND-element into list
       cslorder.source=cslorder.NM_SPECIAL;
-      cslorder.sources=new List;
-      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
-      specialsource->entry=((DNDMsg*)am->extraInfo)->getFE();
+      cslorder.sources=new std::list<NM_specialsourceExt*>;
+      specialsource = new NM_specialsourceExt( am->dndmsg->getFE() );
       specialsource->row=-1;
-      cslorder.sources->addElement(specialsource);
+      cslorder.sources->push_back(specialsource);
     } else if(am->mode==am->AM_MODE_SPECIAL) {
       cslorder.source=cslorder.NM_SPECIAL;
-      cslorder.sources=new List;
-      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
-      specialsource->entry=(FileEntry*)am->extraInfo;
+      cslorder.sources=new std::list<NM_specialsourceExt*>;
+      specialsource = new NM_specialsourceExt( am->getFE() );
       specialsource->row=-1;
-      cslorder.sources->addElement(specialsource);
+      cslorder.sources->push_back(specialsource);
     } else
       cslorder.source=cslorder.NM_ALLENTRIES;
 
     takesamedir=true;
     if(am->mode==am->AM_MODE_DNDACTION) {
-      const char *ddstr=((DNDMsg*)am->extraInfo)->getDestDir();
+      const char *ddstr = am->dndmsg->getDestDir();
       if(ddstr!=NULL) dest=dupstring(ddstr);
       else dest=NULL;
       takesamedir=false;
@@ -192,7 +190,7 @@ CreateSymlinkOp::normalmodecreatesl()
     }
     if(dest!=NULL) _freesafe(dest);
     if(cslorder.source==cslorder.NM_SPECIAL) {
-      if(specialsource!=NULL) _freesafe(specialsource);
+      if ( specialsource != NULL ) delete specialsource;
       delete cslorder.sources;
     }
   }
