@@ -1,7 +1,7 @@
 /* cursorops.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: cursorops.cc,v 1.19 2004/03/18 07:06:29 ralf Exp $ */
+/* $Id: cursorops.cc,v 1.21 2004/12/12 21:57:38 ralf Exp $ */
 
 #include "cursorops.h"
 #include "listermode.h"
@@ -729,7 +729,7 @@ int EnterDirOp::run( WPUContext *wpu, ActionMessage *msg )
   const char *textstr;
 
   if(msg->mode==msg->AM_MODE_DNDACTION) {
-    l1=((DNDMsg*)msg->extraInfo)->getDestLister();
+    l1 = msg->dndmsg->getDestLister();
     if(l1!=NULL) {
       // Element in l1 darstellen
       l1->switch2Mode(0);
@@ -737,8 +737,12 @@ int EnterDirOp::run( WPUContext *wpu, ActionMessage *msg )
       if(lm1!=NULL) {
         if(lm1->isType("NormalMode")==true) {
           nm1=(NormalMode*)lm1;
-          FileEntry *fe=((DNDMsg*)msg->extraInfo)->getFE();
-          if(fe!=NULL) nm1->enterDir(fe->fullname);
+          FileEntry *fe = msg->dndmsg->getFE();
+	  if ( ( fe != NULL ) && ( fe->fullname != NULL ) ) {
+	    tstr = dupstring( fe->fullname );
+            nm1->enterDir( tstr );
+	    _freesafe( tstr );
+	  }
         }
       }
     }
