@@ -1,7 +1,7 @@
 /* datei.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: datei.cc,v 1.90 2004/06/13 21:24:59 ralf Exp $ */
+/* $Id: datei.cc,v 1.102 2004/12/16 19:12:36 ralf Exp $ */
 
 #include "datei.h"
 #include <aguix/lowlevelfunc.h>
@@ -31,10 +31,7 @@
 #include "nmrowdata.h"
 #include "condparser.h"
 #include "stringbuf.h"
-
-#if 0
-#include <string>
-#endif
+#include <aguix/mutex.h>
 
 #ifndef MAXPATHLEN
 #define MAXPATHLEN 2048
@@ -306,6 +303,16 @@ FileEntry::FileEntry()
   contentBuf.size = 0;
   
   outputBuf = NULL;
+
+  customcolors = false;
+  fg[0] = 1;
+  fg[1] = 1;
+  fg[2] = 1;
+  fg[3] = 1;
+  bg[0] = 0;
+  bg[1] = 0;
+  bg[2] = 0;
+  bg[3] = 0;
 }
 
 FileEntry::~FileEntry()
@@ -784,7 +791,7 @@ int Verzeichnis::sort(int mode,bool force)
  return 0;
 }
 
-bool FileEntry::isDir()
+bool FileEntry::isDir() const
 {
   bool dir=false;
   if ( isLink == true ) {
@@ -797,6 +804,7 @@ bool FileEntry::isDir()
 
 FileEntry *FileEntry::duplicate()
 {
+#if 0
   FileEntry *tfe=new FileEntry();
   tfe->fullname=dupstring(fullname);
   tfe->name=dupstring(name);
@@ -810,7 +818,14 @@ FileEntry *FileEntry::duplicate()
 
   memcpy( &( tfe->statbuf ), &statbuf, sizeof( statbuf ) );
   memcpy( &( tfe->dstatbuf ), &dstatbuf, sizeof( dstatbuf ) );
+
+  tfe->setCustomColors( customcolors );
+  tfe->setColor( 0, fg );
+  tfe->setColor( 1, bg );
+
   return tfe;
+#endif
+  return new FileEntry( *this );
 }
 
 bool Datei::fileExists(const char *name)
@@ -931,6 +946,7 @@ int FileEntry::readInfos(bool update)
     statbuf.nlink = stbuf.st_nlink;
     statbuf.blocks = stbuf.st_blocks;
     statbuf.rdev = stbuf.st_rdev;
+    statbuf.dev = stbuf.st_dev;
   } else return 2;
   isCorrupt=false;
   if ( S_ISLNK( statbuf.mode ) ) {
@@ -947,6 +963,7 @@ int FileEntry::readInfos(bool update)
       dstatbuf.nlink = stbuf.st_nlink;
       dstatbuf.blocks = stbuf.st_blocks;
       dstatbuf.rdev = stbuf.st_rdev;
+      dstatbuf.dev = stbuf.st_dev;
     } else {
       /* korrupt */
       isCorrupt=true;
@@ -1153,6 +1170,42 @@ void FileEntry::setLVC( FieldListView *lv, int row, const std::vector<WorkerType
         break;
     }
   }
+  
+  if ( customcolors == true ) {
+    lv->setFG( row, FieldListView::CC_NORMAL, fg[0] );
+    lv->setBG( row, FieldListView::CC_NORMAL, bg[0] );
+    lv->setFG( row, FieldListView::CC_SELECT, fg[1] );
+    lv->setBG( row, FieldListView::CC_SELECT, bg[1] );
+    lv->setFG( row, FieldListView::CC_ACTIVE, fg[2] );
+    lv->setBG( row, FieldListView::CC_ACTIVE, bg[2] );
+    lv->setFG( row, FieldListView::CC_SELACT, fg[3] );
+    lv->setBG( row, FieldListView::CC_SELACT, bg[3] );
+  } else {
+    if ( ( isDir() == true ) && ( isCorrupt == false ) ) {
+      lv->setFG( row, FieldListView::CC_NORMAL, wconfig->getUnselDir( 0 ) );
+      lv->setBG( row, FieldListView::CC_NORMAL, wconfig->getUnselDir( 1 ) );
+      lv->setFG( row, FieldListView::CC_SELECT, wconfig->getSelDir( 0 ) );
+      lv->setBG( row, FieldListView::CC_SELECT, wconfig->getSelDir( 1 ) );
+      lv->setFG( row, FieldListView::CC_ACTIVE, wconfig->getUnselDirAct( 0 ) );
+      lv->setBG( row, FieldListView::CC_ACTIVE, wconfig->getUnselDirAct( 1 ) );
+      lv->setFG( row, FieldListView::CC_SELACT, wconfig->getSelDirAct( 0 ) );
+      lv->setBG( row, FieldListView::CC_SELACT, wconfig->getSelDirAct( 1 ) );
+    } else {
+      lv->setFG( row, FieldListView::CC_NORMAL, wconfig->getUnselFile( 0 ) );
+      lv->setBG( row, FieldListView::CC_NORMAL, wconfig->getUnselFile( 1 ) );
+      lv->setFG( row, FieldListView::CC_SELECT, wconfig->getSelFile( 0 ) );
+      lv->setBG( row, FieldListView::CC_SELECT, wconfig->getSelFile( 1 ) );
+      lv->setFG( row, FieldListView::CC_ACTIVE, wconfig->getUnselFileAct( 0 ) );
+      lv->setBG( row, FieldListView::CC_ACTIVE, wconfig->getUnselFileAct( 1 ) );
+      lv->setFG( row, FieldListView::CC_SELACT, wconfig->getSelFileAct( 0 ) );
+      lv->setBG( row, FieldListView::CC_SELACT, wconfig->getSelFileAct( 1 ) );
+    }
+  }
+  if ( isCorrupt == true ) {
+    lv->setFG( row, FieldListView::CC_NORMAL, 4 );
+    lv->setFG( row, FieldListView::CC_ACTIVE, 4 );
+  }
+
 }
 
 const char *FileEntry::getPermissionString()
@@ -1193,11 +1246,40 @@ const char *FileEntry::getPermissionString()
   return str;
 }
 
-bool FileEntry::isExe()
+bool FileEntry::isExe( bool fastTest ) const
 {
-  if ( ( statbuf.mode & S_IXUSR ) == S_IXUSR ) return true;
-  if ( ( statbuf.mode & S_IXGRP ) == S_IXGRP ) return true;
-  if ( ( statbuf.mode & S_IXOTH ) == S_IXOTH ) return true;
+  if ( fastTest == true ) {
+    if ( ( statbuf.mode & S_IXUSR ) == S_IXUSR ) return true;
+    if ( ( statbuf.mode & S_IXGRP ) == S_IXGRP ) return true;
+    if ( ( statbuf.mode & S_IXOTH ) == S_IXOTH ) return true;
+  } else {
+    uid_t uid;
+    gid_t gid;
+    mode_t m;
+    bool e = false;
+
+    if ( isLink == false ) {
+      uid = userid();
+      gid = groupid();
+      m = mode();
+    } else if ( isCorrupt == false ) {
+      uid = duserid();
+      gid = dgroupid();
+      m = dmode();
+    } else {
+      // corrupt symlink is not executable
+      return false;
+    }
+    
+    if ( uid == geteuid() ) {
+      e = ( ( m & S_IXUSR ) != 0 ) ? true : false;
+    } else if ( ugdb->isInGroup( gid ) == true ) {
+      e = ( ( m & S_IXGRP ) != 0 ) ? true : false;
+    } else {
+      e = ( ( m & S_IXOTH ) != 0 ) ? true : false;
+    }
+    return e;
+  }
   return false;
 }
 
@@ -1221,25 +1303,145 @@ int getDatelen()
   return datelen;
 }
 
-#define FILEOUTPUTLENGTH 4096
+FileEntry::typecandidate_t FileEntry::runTests( WCFiletype *type,
+						bool skipcontent,
+						CondParser *condparser,
+						unsigned char fbuffer[64],
+						int reads )
+{
+  typecandidate_t erg;
+  int minsize, x, y;
+  char *tstr1;
+  
+  if ( type == NULL ) return erg;
+  if ( condparser == NULL ) return erg;
+  
+  if ( skipcontent == false ) {
+    short *filedesc;
+    if ( type->getinternID() == NORMALTYPE ) {
+      if ( type->getUseFiledesc() == true ) {
+	erg.tests_done++;
+	if ( reads > 0 ) {
+	  filedesc = type->getFiledesc();
+	  minsize = 0;
+	  for ( x = 0; x < 64; x++ ) {
+	    if ( filedesc[x] > -1 ) minsize = x + 1;
+	  }
+	  if ( ( reads >= minsize ) && ( minsize > 0 ) ) {
+	    y = 0;
+	    for ( x = 0; x < reads; x++ ) {
+	      if ( filedesc[x] >= 0 ) {
+		if ( filedesc[x] != (short)fbuffer[x] ) {
+		  y = 1;
+		  break;
+		}
+	      }
+	    }
+	    if ( y == 0 ) {
+	      erg.prio = 3;
+	      erg.type = type;
+	    }
+	  }
+	}
+      }
+    }
+  }
+
+  /* if no type found then check extended pattern */
+  if ( erg.type == NULL ) {
+    if ( type->getinternID() == NORMALTYPE ) {
+      if ( type->getUseExtCond() == true ) {
+	erg.tests_done++;
+	condparser->setIgnoreContent( skipcontent );
+	if ( condparser->parse( type->getExtCond(), this ) == 1 ) {
+	  erg.prio = 2;
+	  erg.type = type;
+	}
+      }
+    }
+  }
+
+  /* if no type found then check the pattern */
+  if ( erg.type == NULL ) {
+    if ( type->getinternID() == NORMALTYPE ) {
+      if ( type->getUsePattern() == true ) {
+	erg.tests_done++;
+	if ( type->getPatternUseFullname() == true ) {
+	  tstr1 = fullname;
+	} else {
+	  tstr1 = name;
+	}
+	if ( type->patternMatchString( tstr1 ) == true ) {
+	  erg.prio = 1;
+	  erg.type = type;
+	}
+      }
+    }
+  }
+  return erg;
+}
+
+FileEntry::typecandidate_t FileEntry::checkFiletype( const std::list<WCFiletype*> *ftlist,
+						     bool skipcontent,
+						     CondParser *condparser,
+						     unsigned char fbuffer[64],
+						     int reads )
+{
+  WCFiletype *ft;
+  typecandidate_t cand1, erg, cand2;
+  const std::list<WCFiletype*> *subtypes;
+  std::list<WCFiletype*>::const_iterator it1;
+
+  if ( ftlist == NULL ) return erg;
+  if ( condparser == NULL ) return erg;
+  if ( fbuffer == NULL ) return erg;
+  
+  if ( ftlist->size() < 1 ) {
+    return erg;
+  }
+  if ( ( S_ISDIR( statbuf.mode ) ) && ( isCorrupt == false ) ) {
+    // Wirklich ein Verzeichnis
+    return erg;
+  }
+  
+  for ( it1 = ftlist->begin(); it1 != ftlist->end(); it1++ ) {
+    ft = *it1;
+    cand1 = runTests( ft, skipcontent, condparser, fbuffer, reads );
+    if ( ( cand1.type != NULL ) || ( cand1.tests_done < 1 ) ) {
+      // check subtypes if we had a hit or did no tests
+      subtypes = ft->getSubTypeList();
+      if ( subtypes != NULL ) {
+	if ( subtypes->size() > 0 ) {
+	  cand2 = checkFiletype( subtypes, skipcontent, condparser, fbuffer, reads );
+	  if ( cand2.prio > 0 ) {
+	    // subtypes are always more important
+	    cand1.prio = cand2.prio;
+	    cand1.type = cand2.type;
+	  }
+	}
+      }
+    }
+    if ( cand1.type != NULL ) {
+      if ( cand1.prio > erg.prio ) {
+	erg.prio = cand1.prio;
+	erg.type = cand1.type;
+      }
+    }
+  }
+
+  return erg;
+}
 
 WCFiletype* FileEntry::checkFiletype( List *ftlist, bool skipcontent, CondParser *condparser )
 {
-//printf("start check %s\n",name);
   unsigned char fbuffer[64];
-  int reads,x,y,minsize=0;
+  int reads = 0;
   int id;
   WCFiletype *ft,*unknownft;
-  WCFiletype *foundft = NULL;
-  const char *tstr1;
   CondParser *localparser = NULL;
-
-#if 0
-  char *filestr, *outname, *fileoutput;
-  std::string exestr;
-#endif
-
   PDatei *pfp;
+  typecandidate_t cand1, erg, cand2;
+  const std::list<WCFiletype*> *subtypes;
 
   if ( ftlist == NULL ) return NULL;
   
@@ -1273,7 +1475,6 @@ WCFiletype* FileEntry::checkFiletype( List *ftlist, bool skipcontent, CondParser
   if ( isReg() == false ) skipcontent = true;
   if ( statbuf.size < 1 ) skipcontent = true;
 
-  id=ftlist->initEnum();
   if(skipcontent==false) {
     reads = -1;
     pfp = new PDatei();
@@ -1281,146 +1482,83 @@ WCFiletype* FileEntry::checkFiletype( List *ftlist, bool skipcontent, CondParser
       reads = pfp->read( fbuffer, 64 );
     }
     delete pfp;
-    if ( reads > 0 ) {
-      ft=(WCFiletype*)ftlist->getFirstElement(id);
-      short *filedesc;
-      int fileusage = 0;
-      
-      /* first check only the contect */
-      while(ft!=NULL) {
-        if(ft->getinternID()==NORMALTYPE) {
-          if(ft->getUseFiledesc()==true) {
-            filedesc=ft->getFiledesc();
-            minsize = 0;
-            for(x=0;x<64;x++) {
-              if(filedesc[x]>-1) minsize=x+1;
-            }
-            if ( ( reads >= minsize ) && ( minsize > 0 ) ) {
-              y=0;
-              for(x=0;x<reads;x++) {
-                if(filedesc[x]>=0) {
-                  if(filedesc[x]!=(short)fbuffer[x]) {
-                    y=1;
-                    break;
-                  }
-                }
-              }
-              if(y==0) {
-                foundft=ft;
-                break;
-              }
-            }
-          }
-          if ( ft->getUseFileExp() == true ) {
-            fileusage++;
-          }
-        }
-        ft=(WCFiletype*)ftlist->getNextElement(id);
-      }
-#if 0
-      // this code is functional and does exactly what it should
-      // but it's just too slow and cpu-eating
-      if ( ( foundft == NULL ) && ( fileusage > 0 ) ) {
-        // not found
-
-        outname = Datei::createTMPName();
-        filestr = (char*)_allocsafe( sizeof(char) * ( strlen( FILE_COMMAND ) + strlen( fullname ) + 1 ) );
-        sprintf( filestr, FILE_COMMAND, fullname );
-
-        exestr = filestr;
-        exestr += " > ";
-        exestr += outname;
-        _freesafe( filestr );
-    
-        // get the output of file
-        //TODO: Aufrichtiges Quoting aufpassen
-        system( exestr.c_str() );
-        fileoutput = NULL;
-        fp = fopen( outname, "r" );
-        if ( fp != NULL ) {
-          fileoutput = (char*)_allocsafe( sizeof(char) * FILEOUTPUTLENGTH );
-          reads = fread( fileoutput, 1, FILEOUTPUTLENGTH - 1, fp );
-          fileoutput[reads] = '\0';
-          fclose( fp );
-        }
-    
-        ft = (WCFiletype*)ftlist->getFirstElement( id );
-        while ( ft != NULL ) {
-          if ( ft->getinternID() == NORMALTYPE ) {
-            if ( ft->getUseFileExp() == true ) {
-              if ( ft->fileExpMatchString( fileoutput ) == true ) {
-                foundft = ft;
-                break;
-              }
-            }
-          }
-          ft = (WCFiletype*)ftlist->getNextElement( id );
-        }
-        remove( outname );
-        _freesafe( outname );
-        if ( fileoutput != NULL ) _freesafe( fileoutput );
-      }
-#endif
-    }
   }
 
-  /* if no type found then check extended pattern */
-  if ( foundft == NULL ) {
-    ft = (WCFiletype*)ftlist->getFirstElement( id );
-    while ( ft != NULL ) {
-      if ( ft->getinternID() == NORMALTYPE ) {
-        if ( ft->getUseExtCond() == true ) {
-          if ( condparser == NULL ) {
-            localparser = new CondParser();
-            condparser = localparser;
-          }
-          if ( condparser->parse( ft->getExtCond(), this ) == 1 ) {
-            foundft = ft;
-            break;
-          }
-        }
-      }
-      ft = (WCFiletype*)ftlist->getNextElement( id );
-    }
+  if ( condparser == NULL ) {
+    localparser = new CondParser();
+    condparser = localparser;
   }
 
-  /* if no type found then check the pattern */
-  if ( foundft == NULL ) {
-    ft = (WCFiletype*)ftlist->getFirstElement( id );
-    while ( ft != NULL ) {
-      if ( ft->getinternID() == NORMALTYPE ) {
-        if ( ft->getUsePattern() == true ) {
-          if ( ft->getPatternUseFullname() == true ) {
-            tstr1 = fullname;
-          } else {
-            tstr1 = name;
-          }
-          if ( ft->patternMatchString( tstr1 ) == true ) {
-            foundft = ft;
-            break;
-          }
-        }
+  id = ftlist->initEnum();
+  ft = (WCFiletype*)ftlist->getFirstElement( id );
+  while ( ft != NULL ) {
+    cand1 = runTests( ft, skipcontent, condparser, fbuffer, reads );
+    if ( ( cand1.type != NULL ) || ( cand1.tests_done < 1 ) ) {
+      // check subtypes if we had a hit or did no tests
+      subtypes = ft->getSubTypeList();
+      if ( subtypes != NULL ) {
+	if ( subtypes->size() > 0 ) {
+	  cand2 = checkFiletype( subtypes, skipcontent, condparser, fbuffer, reads );
+	  if ( cand2.prio > 0 ) {
+	    // subtypes are always more important
+	    cand1.prio = cand2.prio;
+	    cand1.type = cand2.type;
+	  }
+	}
       }
-      ft = (WCFiletype*)ftlist->getNextElement( id );
     }
+    if ( cand1.type != NULL ) {
+      if ( cand1.prio > erg.prio ) {
+	erg.prio = cand1.prio;
+	erg.type = cand1.type;
+      }
+    }
+    ft = (WCFiletype*)ftlist->getNextElement( id );
   }
-  if ( foundft == NULL ) {
+
+  if ( erg.type  == NULL ) {
     filetype = unknownft;
   } else {
-    filetype = foundft;
+    filetype = erg.type;
   }
   ftlist->closeEnum( id );
   
+  if ( filetype != NULL ) {
+    ft = filetype;
+    while ( ft->getColorMode() == ft->PARENTCOL ) {
+      ft = ft->getParentType();
+      if ( ft == NULL ) break;
+    }
+    if ( ft != NULL ) {
+      if ( ft->getColorMode() == ft->CUSTOMCOL ) {
+	setCustomColors( true );
+	setColor( 0, ft->getCustomColor( 0 ) );
+	setColor( 1, ft->getCustomColor( 1 ) );
+      } else if ( ft->getColorMode() == ft->EXTERNALCOL ) {
+	char *o1 = condparser->getOutput( ft->getColorExt(), this );
+	int tfg[4], tbg[4], z;
+
+	if ( o1 != NULL ) {
+	  z = sscanf( o1, "%d %d %d %d %d %d %d %d", &tfg[0], &tbg[0], &tfg[1], &tbg[1], &tfg[2], &tbg[2], &tfg[3], &tbg[3] );
+	  if ( z == 8 ) {
+	   setCustomColors( true );
+	   setColor( 0, tfg );
+	   setColor( 1, tbg );
+	  }
+	  _freesafe( o1 );
+	}
+      }
+    }
+  }
+
   // free temporary buffers
   if ( localparser != NULL ) delete localparser;
   freeContentBuf();
   freeOutputBuf();
-//printf("end check\n");
+  
   return filetype;
 }
 
-#undef FILEOUTPUTLENGTH
-
 bool FileEntry::match(char* pattern)
 {
   if(fnmatch(pattern,name,0)==0) return true;
@@ -1904,6 +2042,16 @@ dev_t FileEntry::drdev() const
   return dstatbuf.rdev;
 }
 
+dev_t FileEntry::dev() const
+{
+  return statbuf.dev;
+}
+
+dev_t FileEntry::ddev() const
+{
+  return dstatbuf.dev;
+}
+
 ArrayList *Verzeichnis::getInvalidEntries()
 {
   return invalid_entries;
@@ -2313,16 +2461,16 @@ void FileEntry::readContentBuf( int len )
     reads = -1;
     pfp = new PDatei();
     if ( pfp->open( fullname ) == 0 ) {
-      reads = pfp->read( fbuffer, 64 );
+      reads = pfp->read( fbuffer, len );
     }
     delete pfp;
-    if ( reads > 0 ) {
+    if ( ( reads > 0 ) && ( reads <= len ) ) {
       contentBuf.content = (char*)_allocsafe( len + 1 );
       contentBuf.size = reads;
       for ( i = 0; i < reads; i++ ) {
         contentBuf.content[i] = (char)fbuffer[i];
       }
-      for ( i = reads; i < ( 64 + 1 ); i++ ) {
+      for ( i = reads; i < ( len + 1 ); i++ ) {
         contentBuf.content[i] = '\0';
       }
     }
@@ -2387,3 +2535,146 @@ bool FileEntry::isReg() const
   return false;
 }
 
+void FileEntry::setCustomColors( bool nv )
+{
+  customcolors = nv;
+}
+
+bool FileEntry::getCustomColors() const
+{
+  return customcolors;
+}
+
+void FileEntry::setColor( int m, const int *v )
+{
+  int i;
+
+  if ( v == NULL ) return;
+  if ( m == 0 ) {
+    for ( i = 0; i < 4; i++ ) {
+      fg[i] = v[i];
+    }
+  } else if ( m == 1 ) {
+    for ( i = 0; i < 4; i++ ) {
+      bg[i] = v[i];
+    }
+  }
+}
+
+void FileEntry::setColor( int m, int pos, int v )
+{
+  if ( ( pos >= 0 ) && ( pos < 4 ) ) {
+    if ( m == 0 ) {
+      fg[pos] = v;
+    } else if ( m == 1 ) {
+      bg[pos] = v;
+    }
+  }
+}
+
+int FileEntry::getColor( int m, int pos ) const
+{
+  if ( ( pos >= 0 ) && ( pos < 4 ) ) {
+    if ( m == 0 ) {
+      return fg[pos];
+    } else if ( m == 1 ) {
+      return bg[pos];
+    }
+  }
+  return -1;
+}
+
+const int *FileEntry::getColor( int m ) const
+{
+  if ( m == 0 ) {
+    return fg;
+  } else if ( m == 1 ) {
+    return bg;
+  }
+  return NULL;
+}
+
+/*
+ * Method to check for same destination file
+ * it currently only check the destination itself, not the possible
+ * destination of a symlink
+ */
+bool FileEntry::isSameFile( const char *othername, bool follow_symlinks ) const
+{
+  struct stat buf;
+
+  if ( othername == NULL ) return false;
+  if ( stat( othername, &buf ) != 0 ) return false;
+
+  if ( ( isLink == true ) && ( follow_symlinks == true ) && ( isCorrupt == false ) ) {
+    if ( ( dinode() == buf.st_ino ) && ( ddev() == buf.st_dev ) ) return true;
+  } else {
+    if ( ( inode() == buf.st_ino ) && ( dev() == buf.st_dev ) ) return true;
+  }
+  return false;
+}
+
+FileEntry::FileEntry( const FileEntry &other )
+{
+  fullname = ( other.fullname != NULL ) ? dupstring( other.fullname ) : NULL;
+  name = (other.name != NULL ) ? dupstring( other.name ) : NULL;
+  filetype = other.filetype;
+  use = other.use;
+  isHidden = other.isHidden;
+  reclistQueued = other.reclistQueued;
+
+  memcpy( &statbuf, &( other.statbuf ), sizeof( statbuf ) );
+  memcpy( &dstatbuf, &( other.dstatbuf ), sizeof( dstatbuf ) );
+  customcolors = other.customcolors;
+  setColor( 0, other.fg );
+  setColor( 1, other.bg );
+
+  dirsize = other.dirsize;
+  select = other.select;
+  isLink = other.isLink;
+  isCorrupt = other.isCorrupt;
+  nr = other.nr;
+  inFilter = other.inFilter;
+
+  // not copied
+  contentBuf.content = NULL;
+  contentBuf.size = 0;
+  
+  outputBuf = NULL;
+}
+
+/*
+ * Method to test for equal fileentry
+ *
+ * Does same extented test to ensure that the fileentry
+ * is a duplicated version
+ */
+bool FileEntry::equals( const FileEntry *fe ) const
+{
+  if ( fe == NULL ) return false;
+  if ( fe == this ) return true;
+  return equals( *fe );
+}
+
+bool FileEntry::equals( const FileEntry &fe ) const
+{
+  // require name
+  if ( ( fullname == NULL ) || ( fe.fullname == NULL ) ) return false;
+  
+  // require same inode/device
+  if ( statbuf.inode == 0 ) return false;
+  if ( ( statbuf.inode != fe.statbuf.inode ) || ( statbuf.dev != fe.statbuf.dev ) ) return false;
+
+  // require same size
+  if ( statbuf.size != fe.statbuf.size ) return false;
+
+  // require same name
+  if ( strcmp( fullname, fe.fullname ) != 0 ) return false;
+
+  // finally require same times
+  if ( statbuf.lastaccess != fe.statbuf.lastaccess ) return false;
+  if ( statbuf.lastmod != fe.statbuf.lastmod ) return false;
+  if ( statbuf.lastchange != fe.statbuf.lastchange ) return false;
+
+  return true;
+}
