@@ -1,7 +1,7 @@
 /* deleteop.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: deleteop.cc,v 1.25 2004/03/18 07:06:40 ralf Exp $ */
+/* $Id: deleteop.cc,v 1.27 2004/12/12 21:57:24 ralf Exp $ */
 
 #include "listermode.h"
 #include "normalmode.h"
@@ -87,7 +87,7 @@ DeleteOp::normalmodedelete()
   struct NM_deleteorder delorder;
   NormalMode *nm1=NULL;
   ListerMode *lm1=NULL;
-  struct NM_specialsource *specialsource=NULL;
+  NM_specialsourceExt *specialsource=NULL;
   
   if(startlister==NULL) return 1;
   lm1=startlister->getActiveMode();
@@ -109,7 +109,7 @@ DeleteOp::normalmodedelete()
     nm1->deletef(&delorder);
     delete delorder.dowin;
     if(delorder.source==delorder.NM_SPECIAL) {
-      if(specialsource!=NULL) _freesafe(specialsource);
+      if ( specialsource != NULL ) delete specialsource;
       delete delorder.sources;
     }
   }
