@@ -1,7 +1,7 @@
 /* import.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: import.cc,v 1.17 2004/03/18 07:09:55 ralf Exp $ */
+/* $Id: import.cc,v 1.19 2004/10/27 21:07:39 ralf Exp $ */
 
 #include "import.h"
 #include "worker_commands.h"
@@ -2184,12 +2184,12 @@ int WConfig::loadBinFiletype( Datei *fh, WCFiletype *f1 )
         len = fh->getInt();
         chunksize -= Datei::getIntSize();
         
-        str = fh->getString( len );
+        str = fh->getString( len ); // was fileExp
         chunksize -= len;
-        f1->setFileExp( str );
         _freesafe(str);
         
-        f1->setUseFileExp( ( fh->getUShort() == 1 ) ? true : false );
+	// was useFileExp
+        fh->getUShort();
         chunksize -= Datei::getUShortSize();
         
         // new >2.5.0
