@@ -1,7 +1,7 @@
 /* informationmode.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: informationmode.cc,v 1.31 2004/03/18 07:10:06 ralf Exp $ */
+/* $Id: informationmode.cc,v 1.34 2004/12/15 22:46:02 ralf Exp $ */
 
 #include "informationmode.h"
 #include "normalmode.h"
@@ -44,7 +44,7 @@ InformationMode::InformationMode(Lister *parent):ListerMode(parent)
 
 InformationMode::~InformationMode()
 {
-  lastactivefe=NULL;
+  clearLastActiveFE();
 }
 
 void InformationMode::messageHandler(AGMessage *msg)
@@ -96,7 +96,7 @@ void InformationMode::off()
   lv = NULL;
   parentlister->setActiveMode(NULL);
   parentlister->setName("");
-  lastactivefe=NULL;
+  clearLastActiveFE();
 }
 
 void InformationMode::reconf(int tx,int ty,int tw,int th)
@@ -192,9 +192,10 @@ void InformationMode::update(bool force)
   NormalMode *nm1=NULL;
   ListerMode *lm1=NULL;
   Lister *ol=NULL;
-  List *files;
-  NM_specialsource *ss1;
-  FileEntry *fe;
+  std::list<NM_specialsourceExt*> *files;
+  std::list<NM_specialsourceExt*>::iterator it1;
+  NM_specialsourceExt *ss1;
+  const FileEntry *fe;
   
   ol=parentlister->getWorker()->getOtherLister(parentlister);
   if(ol!=NULL) {
@@ -206,31 +207,35 @@ void InformationMode::update(bool force)
   
   if ( ( force == true ) ||
        ( nospaceinfo == true ) )
-    lastactivefe=NULL;
+    clearLastActiveFE();
   
   if(nm1!=NULL) {
-    files=new List();
+    files=new std::list<NM_specialsourceExt*>;
     
-    nm1->getSelFiles(files,NormalMode::NM_GETFILES_ONLYACTIVE);
-    ss1=(NM_specialsource*)files->getFirstElement();
+    nm1->getSelFiles( files, NormalMode::NM_GETFILES_ONLYACTIVE );
+    if ( files->begin() != files->end() )
+      ss1 = *( files->begin() );
+    else
+      ss1 = NULL;
     if(ss1!=NULL) {
-      fe=ss1->entry;
-      if(fe!=lastactivefe) {
-        lastactivefe=fe;
-        showinfos();
+      fe = ss1->entry();
+      if ( fe->equals( lastactivefe ) == false ) {
+	clearLastActiveFE();
+	if ( fe != NULL ) {
+	  lastactivefe = new FileEntry( *fe );
+	}
+	showinfos();
       }
     } else {
       if ( ol->isActive() == true ) {
         // only clear LV when other side is active and has no active fe
-        lastactivefe=NULL;
+	clearLastActiveFE();
         showinfos();
       }
     }
     
-    ss1=(NM_specialsource*)files->getFirstElement();
-    while(ss1!=NULL) {
-      _freesafe(ss1);
-      ss1=(NM_specialsource*)files->getNextElement();
+    for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
+      delete *it1;
     }
     
     delete files;
@@ -335,6 +340,7 @@ void InformationMode::showinfos()
 
     lv->setText( 4, 0, "" );
     if ( fe->isCorrupt == false ) {
+      // dmode is only accessed when not corrupt
       if ( S_ISREG( ( fe->isLink == true ) ? fe->dmode() : fe->mode() ) ) {
         enum PDatei::pdatei_handler pt;
         
@@ -551,3 +557,9 @@ void InformationMode::showinfos()
   lv->redraw();
 }
 
+void InformationMode::clearLastActiveFE() {
+  if ( lastactivefe != NULL ) {
+    delete lastactivefe;
+    lastactivefe = NULL;
+  }
+}
