@@ -1,7 +1,7 @@
 /* normalmode.h
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: normalmode.h,v 1.94 2004/06/13 21:32:35 ralf Exp $ */
+/* $Id: normalmode.h,v 1.98 2004/12/12 22:18:53 ralf Exp $ */
 
 #ifndef NORMALMODE_H
 #define NORMALMODE_H
@@ -100,10 +100,50 @@ public:
   FileEntry *firstnullft;
 };
 
+class NM_CopyOp_Dir
+{
+public:
+  NM_CopyOp_Dir(const FileEntry *fe);
+  ~NM_CopyOp_Dir();
+  NM_CopyOp_Dir( const NM_CopyOp_Dir &other );
+  NM_CopyOp_Dir &operator=( const NM_CopyOp_Dir &other );
+
+  int createSubDirs(struct NM_copyorder*copyorder,unsigned long *gf,unsigned long *gd);
+  int createSubDirs(struct NM_deleteorder*delorder,unsigned long *gf,unsigned long *gd);
+  int createSubDirs();
+  
+  List *subdirs; // contains elements of this class
+  Verzeichnis *verz;
+  const FileEntry *fileentry;
+  unsigned long files,dirs;
+  loff_t bytes;
+  bool ok; // only use this dir if ok==true
+  bool user_abort; // true when user selected cancel from one of
+                   // the requesters used inside
+  
+  unsigned long error_counter; // files/dirs not finished (recursive)
+};
+
+/*
+ * NM_specialsourceExt always stores a duplicate FE
+ */
+class NM_specialsourceExt {
+ public:
+  NM_specialsourceExt( const FileEntry *tfe );
+  ~NM_specialsourceExt();
+  const FileEntry *entry() const
+  {
+    return fe;
+  }
+  int row;  // corresponding row or -1
+ private:
+  FileEntry *fe;  // element to copy
+};
+
 struct NM_copyorder {
   char *destdir;
   enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
-  List *sources; // contains elements of struct NM_specialsource
+  std::list<NM_specialsourceExt*> *sources;
   // modes:
   bool move; // true = move, false = copy
   bool follow_symlinks;
@@ -123,7 +163,7 @@ struct NM_copyorder {
 
 struct NM_deleteorder {
   enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL,NM_ONLYSELECTED} source;
-  List *sources; // contains elements of struct NM_specialsource
+  std::list<NM_specialsourceExt*> *sources;
   enum {NM_DELMODE_NORMAL,NM_DELMODE_NODATABASE} delmode;
   enum {NM_DIRDELETE_NORMAL,NM_DIRDELETE_ALL,NM_DIRDELETE_NONE} dirdel;
   DeleteOpWin *dowin;
@@ -151,7 +191,7 @@ public:
   char *view_str;
 
   enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
-  List *sources; // contains elements of struct NM_specialsource
+  std::list<NM_specialsourceExt*> *sources;
   
   // this is for DND-actions for identifying the destination (for the dp-flag)
   class NormalMode *destmode;
@@ -160,7 +200,7 @@ public:
 };
 
 struct NM_startprogorder {
-  FileEntry *usefe;
+  const FileEntry *usefe;
   bool global;
   bool inbackground;
   enum {NM_START_NORMAL=0,
@@ -173,42 +213,42 @@ struct NM_startprogorder {
 
 struct NM_renameorder {
   enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
-  List *sources; // contains elements of struct NM_specialsource
+  std::list<NM_specialsourceExt*> *sources;
 };
 
 struct NM_dirsizeorder {
   enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
-  List *sources; // contains elements of struct NM_specialsource
+  std::list<NM_specialsourceExt*> *sources;
 };
 
 struct NM_createsymlinkorder {
   bool local;
   char *destdir;
   enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
-  List *sources; // contains elements of struct NM_specialsource
+  std::list<NM_specialsourceExt*> *sources;
 };
 
 struct NM_chmodorder {
   bool ondirs,onfiles,recursive;
   enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
-  List *sources; // contains elements of struct NM_specialsource
+  std::list<NM_specialsourceExt*> *sources;
 };
 
 struct NM_chownorder {
   bool ondirs,onfiles,recursive;
   enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
-  List *sources; // contains elements of struct NM_specialsource
+  std::list<NM_specialsourceExt*> *sources;
 };
 
 struct NM_changesymlinkorder {
   enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
-  List *sources; // contains elements of struct NM_specialsource
+  std::list<NM_specialsourceExt*> *sources;
 };
 
 class NM_extern_fe {
 public:
   NM_extern_fe(char *nfn);
-  NM_extern_fe(char *nfn,FileEntry *fe, int row );
+  NM_extern_fe( char *nfn, const FileEntry *fe, int row );
   ~NM_extern_fe();
   NM_extern_fe( const NM_extern_fe &other );
   NM_extern_fe &operator=( const NM_extern_fe &other );
@@ -217,7 +257,7 @@ public:
   char *getName(bool noext);
   void setDirFinished(int);
   int getDirFinished();
-  FileEntry *getFE();
+  const FileEntry *getFE();
   int getRow();
 protected:
   char *fullname[2];
@@ -227,36 +267,6 @@ protected:
   int dirfinished;
 };
 
-class NM_CopyOp_Dir
-{
-public:
-  NM_CopyOp_Dir(const FileEntry *fe);
-  ~NM_CopyOp_Dir();
-  NM_CopyOp_Dir( const NM_CopyOp_Dir &other );
-  NM_CopyOp_Dir &operator=( const NM_CopyOp_Dir &other );
-
-  int createSubDirs(struct NM_copyorder*copyorder,unsigned long *gf,unsigned long *gd);
-  int createSubDirs(struct NM_deleteorder*delorder,unsigned long *gf,unsigned long *gd);
-  int createSubDirs();
-  
-  List *subdirs; // contains elements of this class
-  Verzeichnis *verz;
-  const FileEntry *fileentry;
-  unsigned long files,dirs;
-  loff_t bytes;
-  bool ok; // only use this dir if ok==true
-  bool user_abort; // true when user selected cancel from one of
-                   // the requesters used inside
-  
-  unsigned long error_counter; // files/dirs not finished (recursive)
-};
-
-struct NM_specialsource {
-  FileEntry *entry;  // element to copy
-  int row;  // corresponding row or -1
-  NM_CopyOp_Dir *cod; // when fe is dir, this will be the corresponding structure
-};
-
 class NMExtList
 {
 public:
@@ -349,14 +359,14 @@ public:
   typedef enum {NM_GETFILES_ONLYACTIVE,
                 NM_GETFILES_ONLYSELECT,
                 NM_GETFILES_SELORACT} nm_getfiles_t;
-  int getSelFiles(List*,nm_getfiles_t);
-  int getSelFiles(List*,nm_getfiles_t,bool unselect);
+  int getSelFiles( std::list<NM_specialsourceExt*>*, nm_getfiles_t, bool unselect = false );
+  void freeSelFiles( std::list<NM_specialsourceExt*>* ) const;
   int getDirSize( FileEntry *fe, loff_t &return_size );
 
   virtual bool startdnd(DNDMsg *dm);
   virtual bool isyours(GUIElement *elem);
 
-  FileEntry *getFE4DNDSTART(AGDNDSTART *st);
+  FileEntry *getFE4DNDSTART( const AGDNDSTART *st );
   void setSortmode(int nmode);
   
   typedef enum {NM_FILTER_INCLUDE,NM_FILTER_EXCLUDE,NM_FILTER_UNSET} nm_filter_t;
@@ -370,7 +380,7 @@ public:
   static void freeExtList( List *extlist );
   char *parseComStrExt(char *sstr,class NM_externorder *extorder,int maxlen, NMExtList *entries[2], bool quote);
   /* deselect for extern access, check for existance of fe => slow for big dirs */
-  void deselect( FileEntry *fe );
+  void deselect( const FileEntry *fe );
   int createExtList( ActionMessage *amsg, bool rec, List *extlist );
   static int getEntriesRealSize( List* );
   virtual void lvbDoubleClicked();
@@ -416,6 +426,23 @@ public:
                          const mode_t wanted_mode,
                          struct NM_copyorder *copyorder );
 protected:
+  /*
+   * NM_specialsourceInt always stores a pointer to a FE
+   */
+  class NM_specialsourceInt {
+   public:
+    NM_specialsourceInt( FileEntry *tfe );
+    ~NM_specialsourceInt();
+    FileEntry *entry() const
+    {
+      return fe;
+    }
+    int row;  // corresponding row or -1
+    NM_CopyOp_Dir *cod; // when fe is dir, this will be the corresponding structure
+   private:
+    FileEntry *fe;  // element to copy
+  };
+
   void reconf( int nx, int ny, int nw, int nh, bool force = false );
   void buildLister();
   void deleteCache();
@@ -476,7 +503,7 @@ protected:
   bool searchmode_ignore_case;
   char *oldstr;
   char *lastsearch;
-  List *copy_deletefe_list,*copy_lvc_list;
+  std::list<const FileEntry*> *copy_deletefe_list;
   int sortmode;
   int filterlfdnr;
   
@@ -489,15 +516,15 @@ protected:
   struct timeval lastclick;
   int lastrow;
   void removeEntry(FileEntry *fe, int row,bool quick);
-  void fixSpecialSourceList( List *l, int deleted_row );
-  int createExtList(List *splist,bool rec,List *extlist);
+  void fixSpecialSourceList( const std::list<NM_specialsourceInt*> *l, int deleted_row );
+  int createExtList( std::list<NM_specialsourceExt*> *splist,bool rec,List *extlist) const;
   int create_externfe_rec(List *extlist,
                           char *fullname,
                           bool rec,
-                          FileEntry *fe,
-                          int row );
+                          const FileEntry *fe,
+                          int row ) const;
   char *parseComStr(char *sstr,class NM_externorder *extorder,int maxlen, NMExtList *entries[2], bool quote );
-  void deselect(FileEntry *fe, int row );
+  void deselect( FileEntry *fe, int row );
   char *StringRequest(char *sstr,class NM_externorder *extorder,int maxlen, NMExtList *entries[2]);
   void startsearchmode();
   void finishsearchmode();
@@ -510,20 +537,20 @@ protected:
     bool forAll;
     mode_t newmode;
   } changemod_info_t;
-  int worker_changemod( const struct NM_specialsource *ss1,
+  int worker_changemod( const NM_specialsourceInt *ss1,
                         changemod_info_t *cminfo,
                         const NM_chmodorder *cmorder );
-  int applyNewMode( const NM_specialsource *ss1, mode_t newmode );
+  int applyNewMode( const NM_specialsourceInt *ss1, mode_t newmode );
   typedef struct changeown_info{
     changeown_info();
     bool forAll;
     uid_t newuid;
     gid_t newgid;
   } changeown_info_t;
-  int worker_changeown( const struct NM_specialsource *ss1,
+  int worker_changeown( const struct NM_specialsourceInt *ss1,
                         changeown_info_t *coinfo,
                         const NM_chownorder *coorder );
-  int applyNewOwner( const NM_specialsource *ss1, uid_t newuid, gid_t newgid );
+  int applyNewOwner( const NM_specialsourceInt *ss1, uid_t newuid, gid_t newgid );
 
   int requestNewMode(FileEntry *fe,mode_t *return_mode);
   void setFilters(List*);
@@ -634,11 +661,29 @@ protected:
     // contains the name and the position of the ft in the wconfig list
     // index of ft because so every slave can hold a copy of the ft-list
     // to work without need to lock
-    typedef struct _ft_recres_list_t {
+    class ft_recres_list_t {
+    public:
       char *name;
-      int ft_index;
-      struct _ft_recres_list_t *next;
-    } ft_recres_list_t;
+      std::vector<unsigned int> *ft_index;
+      bool customcolors;
+      int *fg, *bg;
+      ft_recres_list_t *next;
+
+      ft_recres_list_t()
+      {
+	name = NULL;
+	ft_index = NULL;
+	customcolors = false;
+	fg = bg = NULL;
+	next = NULL;
+      }
+      ~ft_recres_list_t()
+      {
+	if ( fg != NULL ) delete [] fg;
+	if ( bg != NULL ) delete [] bg;
+        if ( name != NULL ) _freesafe( name );
+      }
+    };
     
     ft_recres_list();
     ~ft_recres_list();
@@ -664,7 +709,21 @@ protected:
 
   void ft_list_clear();
   void ft_list_update();
-  int slave_checkFiletype( const char *fullname, bool dontCheckContent );
+
+  class checkFiletypeRes_t {
+  public:
+    std::vector<unsigned int> *v;
+    bool customcolors;
+    int *fg, *bg;
+    
+    checkFiletypeRes_t() {
+      v = NULL;
+      fg = bg = NULL;
+      customcolors = false;
+    }
+  };
+  
+  checkFiletypeRes_t slave_checkFiletype( const char *fullname, bool dontCheckContent );
 #endif
   
   bool lasteagain;
@@ -687,11 +746,10 @@ protected:
   List *chownGroupList;
   
   bool isColumnVisible( const WorkerTypes::listcol_t c );
-  bool safeParse;
   bool visChanged;
   int oldlvy;
   FileEntry *findFileEntryForData( int data, int startpos );
-  NM_specialsource *getSpecialsourceForFE( FileEntry *searchfe );
+  NM_specialsourceExt *getSpecialsourceForFE( const FileEntry *searchfe );
   
   void showInvalidList( ArrayList *list, const char *descr );
   void changeSortModeForField( int field );
@@ -700,6 +758,9 @@ protected:
   time_t lastBusyFlagUpdate;
   
   CondParser condparser;
+  WCFiletype *findFiletype( std::vector<unsigned int> *v );
+  int getSelFiles( std::list<NM_specialsourceInt*> *, nm_getfiles_t, bool unselect );
+  FileEntry *findFE( const FileEntry *tfe );
 };
 
 class NMFieldListViewDND:public FieldListViewDND
