@@ -1,7 +1,7 @@
 /* ownop.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: ownop.cc,v 1.30 2004/03/18 07:10:43 ralf Exp $ */
+/* $Id: ownop.cc,v 1.32 2004/12/12 21:52:03 ralf Exp $ */
 
 #include "ownop.h"
 #include "listermode.h"
@@ -140,14 +140,14 @@ OwnOp::normalmodeown()
   NormalMode *nm1=NULL,
              *nm2=NULL;
   ListerMode *lm1=NULL;
-  struct NM_specialsource *specialsource=NULL;
+  NM_specialsourceExt *specialsource=NULL;
   
   if(am->mode==am->AM_MODE_DNDACTION) {
-    lm1=((DNDMsg*)am->extraInfo)->getSourceMode();
+    lm1 = am->dndmsg->getSourceMode();
     if(lm1==NULL) return 1;
     if(lm1->isType("NormalMode")!=true) return 1;
     nm1=(NormalMode*)lm1;
-    lm1=((DNDMsg*)am->extraInfo)->getDestMode();
+    lm1 = am->dndmsg->getDestMode();
     if(lm1!=NULL)
       if(lm1->isType("NormalMode")==true)
         nm2=(NormalMode*)lm1;
@@ -172,19 +172,17 @@ OwnOp::normalmodeown()
   else if(am->mode==am->AM_MODE_DNDACTION) {
     // insert DND-element into list
     extorder.source=extorder.NM_SPECIAL;
-    extorder.sources=new List;
-    specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
-    specialsource->entry=((DNDMsg*)am->extraInfo)->getFE();
+    extorder.sources=new std::list<NM_specialsourceExt*>;
+    specialsource = new NM_specialsourceExt( am->dndmsg->getFE() );
     specialsource->row=-1;
-    extorder.sources->addElement(specialsource);
+    extorder.sources->push_back(specialsource);
     extorder.destmode=nm2;
   } else if(am->mode==am->AM_MODE_SPECIAL) {
     extorder.source=extorder.NM_SPECIAL;
-    extorder.sources=new List;
-    specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
-    specialsource->entry=(FileEntry*)am->extraInfo;
+    extorder.sources=new std::list<NM_specialsourceExt*>;
+    specialsource = new NM_specialsourceExt( am->getFE() );
     specialsource->row=-1;
-    extorder.sources->addElement(specialsource);
+    extorder.sources->push_back(specialsource);
   } else
     extorder.source=extorder.NM_ALLENTRIES;
 
@@ -213,7 +211,7 @@ OwnOp::normalmodeown()
   nm1->externprog(&extorder);
 
   if(extorder.source==extorder.NM_SPECIAL) {
-    if(specialsource!=NULL) _freesafe(specialsource);
+    if ( specialsource != NULL ) delete specialsource;
     delete extorder.sources;
   }
 
