@@ -1,7 +1,7 @@
 /* reloadop.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: reloadop.cc,v 1.16 2004/03/18 07:10:58 ralf Exp $ */
+/* $Id: reloadop.cc,v 1.18 2004/12/12 21:46:17 ralf Exp $ */
 
 #include "reloadop.h"
 #include "listermode.h"
@@ -122,7 +122,7 @@ ReloadOp::normalmodereload()
   switch(reloadside) {
     case RELOADOP_OTHERSIDE:
       if(am->mode==am->AM_MODE_DNDACTION) {
-        tl=((DNDMsg*)am->extraInfo)->getDestLister();
+        tl = am->dndmsg->getDestLister();
       } else {
         if(startlister==ll) tl=lr;
         else tl=ll;
@@ -136,7 +136,7 @@ ReloadOp::normalmodereload()
       break;
     default:
       if(am->mode==am->AM_MODE_DNDACTION) {
-        tl=((DNDMsg*)am->extraInfo)->getSourceLister();
+        tl = am->dndmsg->getSourceLister();
       } else tl=startlister;
       break;
   }
