@@ -1,7 +1,7 @@
 /* showimagemode.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: showimagemode.cc,v 1.16 2004/03/18 07:11:54 ralf Exp $ */
+/* $Id: showimagemode.cc,v 1.18 2004/12/12 21:28:29 ralf Exp $ */
 
 #include "showimagemode.h"
 #include "normalmode.h"
@@ -47,7 +47,7 @@ ShowImageMode::ShowImageMode(Lister *parent):ListerMode(parent)
 
 ShowImageMode::~ShowImageMode()
 {
-  lastactivefe=NULL;
+  clearLastActiveFE();
   _freesafe(show_program);
 }
 
@@ -85,7 +85,7 @@ void ShowImageMode::off()
   imgwin->close();
   delete imgwin;
   imgwin=NULL;
-  lastactivefe=NULL;
+  clearLastActiveFE();
 }
 
 void ShowImageMode::reconf(int tx,int ty,int tw,int th)
@@ -327,11 +327,12 @@ void ShowImageMode::update(bool force)
   NormalMode *nm1=NULL;
   ListerMode *lm1=NULL;
   Lister *ol=NULL;
-  List *files;
+  std::list<NM_specialsourceExt*> *files;
+  std::list<NM_specialsourceExt*>::iterator it1;
   List *colors;
   WC_Color *bgcol;
-  NM_specialsource *ss1;
-  FileEntry *fe;
+  NM_specialsourceExt *ss1;
+  const FileEntry *fe;
   char *buf,*colbuf, *buf2;
   std::string str1;
   
@@ -343,17 +344,23 @@ void ShowImageMode::update(bool force)
         nm1=(NormalMode*)lm1;
   }
   
-  if(force==true) lastactivefe=NULL;
+  if(force==true) clearLastActiveFE();
   
   if(nm1!=NULL) {
-    files=new List();
+    files=new std::list<NM_specialsourceExt*>;
     
-    nm1->getSelFiles(files,NormalMode::NM_GETFILES_ONLYACTIVE);
-    ss1=(NM_specialsource*)files->getFirstElement();
+    nm1->getSelFiles( files, NormalMode::NM_GETFILES_ONLYACTIVE );
+    if ( files->begin() != files->end() )
+      ss1 = *( files->begin() );
+    else
+      ss1 = NULL;
     if(ss1!=NULL) {
-      fe=ss1->entry;
-      if(fe!=lastactivefe) {
-        lastactivefe=fe;
+      fe = ss1->entry();
+      if ( fe->equals( lastactivefe ) == false ) {
+	clearLastActiveFE();
+	if ( fe != NULL ) {
+	  lastactivefe = new FileEntry( *fe );
+	}
         if(lastactivefe!=NULL) {
           // first get the background color in the X format for xli
           colors=(List*)wconfig->getColors();
@@ -396,14 +403,12 @@ void ShowImageMode::update(bool force)
         }
       }
     } else {
-      lastactivefe=NULL;
+      clearLastActiveFE();
       blank();
     }
     
-    ss1=(NM_specialsource*)files->getFirstElement();
-    while(ss1!=NULL) {
-      _freesafe(ss1);
-      ss1=(NM_specialsource*)files->getNextElement();
+    for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
+      delete *it1;
     }
     
     delete files;
@@ -533,3 +538,9 @@ char *ShowImageMode::requestDefaultViewer()
   return result;
 }
 
+void ShowImageMode::clearLastActiveFE() {
+  if ( lastactivefe != NULL ) {
+    delete lastactivefe;
+    lastactivefe = NULL;
+  }
+}
