@@ -1,7 +1,7 @@
 /* wconfig.h
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: wconfig.h,v 1.49 2004/06/13 21:36:11 ralf Exp $ */
+/* $Id: wconfig.h,v 1.53 2004/10/27 21:09:40 ralf Exp $ */
 
 #ifndef WCONFIG_H
 #define WCONFIG_H
@@ -574,11 +574,6 @@ public:
   void setinternID(int);
   int getNameLen();
 
-  void setFileExp( const char* );
-  const char *getFileExp();
-  void setUseFileExp( bool );
-  bool getUseFileExp();
-  bool fileExpMatchString( const char * );
   void setPatternIgnoreCase( bool nv );
   bool getPatternIgnoreCase();
   void setPatternUseRegExp( bool nv );
@@ -596,6 +591,25 @@ public:
   void setUseExtCond( bool );
   const char *getExtCond() const;
   bool getUseExtCond() const;
+  void addSubType( WCFiletype *newtype );
+  void removeSubType( WCFiletype *remtype );
+  const std::list<WCFiletype*> *getSubTypeList();
+  void setParentType( WCFiletype *pt );
+  WCFiletype *getParentType() const;
+  std::vector<unsigned int> *getTypePos();
+  void sortSubTypeList();
+  static bool sortfunctionType( WCFiletype *ft1, WCFiletype *ft2 );
+
+  enum colordef_t {
+    DEFAULTCOL, CUSTOMCOL, EXTERNALCOL, PARENTCOL
+  };
+  colordef_t getColorMode() const;
+  void setColorMode( colordef_t newmode );
+  const int *getCustomColor( int m ) const;
+  void setCustomColor( int m, const int *c );
+  void setCustomColor( int m, int pos, int c );
+  const char *getColorExt() const;
+  void setColorExt( const char *s );
 protected:
   char *name;
   char *pattern;
@@ -615,16 +629,6 @@ protected:
   int internID;
   int namelen;
   
-  bool usefileexp;
-  char *fileexp;
-
-#if 0  // for filetype checking with file
-  regex_t preg;
-  bool regex_compiled;
-  void free_compexp();
-  void compile_exp();
-#endif
-
   bool patternIgnoreCase;
   bool patternUseRegExp;
   bool patternUseFullname;
@@ -637,6 +641,14 @@ protected:
   void freeCompPatternReg();
   void compilePatternReg();
 #endif
+
+  std::list<WCFiletype*> *subtypes;
+  WCFiletype *parentType;
+  void getTypePos( const WCFiletype *child, std::vector<unsigned int> *v );
+
+  colordef_t colmode;
+  int colfg[4], colbg[4];
+  char *colext;
 };
 
 class WCHotkey:public WCKey
