@@ -1,7 +1,7 @@
 /* worker.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: worker.cc,v 1.102 2004/03/18 07:12:28 ralf Exp $ */
+/* $Id: worker.cc,v 1.104 2004/10/27 21:09:56 ralf Exp $ */
 
 #include "worker.h"
 #include "lister.h"
@@ -44,6 +44,7 @@
 #include "configheader.h"
 #include "configparser.tab.h"
 #include "worker_shm.h"
+#include "parentactionop.h"
 
 Requester *Worker::req_static=NULL;
 int Worker::argc = 0;
@@ -128,6 +129,8 @@ Worker::Worker(int targc,char **targv)
   keyhashtable = NULL;
   keyhashtable_size = 1000;
   waitcursorcount = 0;
+  restart = false;
+  configstarted = false;
 }
 
 Worker::~Worker()
@@ -159,13 +162,13 @@ void Worker::run()
   int i,j;
   time_t lastConfigCheck;
   bool found;
-  bool restart=false;
-  bool configstarted=false;
   int waitcount=0;
   int erg;
   time_t lastConfig = 0;
   std::string str1;
 
+  restart = false;
+  configstarted = false;
   wconfig->loadsize(&w,&h);
   if ( w < 300 ) w = 300;
   if ( h < 200 ) h = 200;
@@ -231,9 +234,8 @@ void Worker::run()
             about();
             handleSub=false;
           } else if(msg->button.button==configureb) {
-            restart=configure();
+            configure();
             handleSub=false;
-            configstarted=true;
           } else if(msg->button.button==clockbar) {
             if( msg->button.state == 1 ) {
               shuffleButton( 1 );
@@ -392,14 +394,14 @@ int main(int argc,char **argv)
   }
   if(showversion==true) {
     printf("Worker Version: %d.%d.%d%s\n",WORKER_MAJOR,WORKER_MINOR,WORKER_PATCH,WORKER_COMMENT);
-    printf("  Author: Ralf Hoffmann <ralf.hoffmann@epost.de>\n");
+    printf("  Author: Ralf Hoffmann <ralf@boomerangsworld.de>\n");
     printf("  Homepage: http://www.boomerangsworld.de/worker\n\n");
     exit(0);
   }
   if(showhelp==true) {
     printf("Worker Filemanager\n");
     printf("  Version: %d.%d.%d%s\n",WORKER_MAJOR,WORKER_MINOR,WORKER_PATCH,WORKER_COMMENT);
-    printf("  Author: Ralf Hoffmann <ralf.hoffmann@epost.de>\n");
+    printf("  Author: Ralf Hoffmann <ralf@boomerangsworld.de>\n");
     printf("  Homepage: http://www.boomerangsworld.de/worker\n\n");
     printf("  Options: -V, --version\tPrint the program version and exit\n");
     printf("           -h, --help\tPrint this help and exit\n\n");
@@ -974,7 +976,7 @@ void Worker::about()
 #ifdef DEBUG
   str1 += "|This program is compiled as debugging version|  If you aren't a developer, please notice the author about it!";
 #endif
-  str1 += "||Homepage: http://www.boomerangsworld.de/worker|EMail: Ralf Hoffmann <ralf.hoffmann@epost.de>";
+  str1 += "||Homepage: http://www.boomerangsworld.de/worker|EMail: Ralf Hoffmann <ralf@boomerangsworld.de>";
   str1 += "||Released under the GNU General Public License:||";
   str1 += "This program is free software; you can redistribute it and/or modify|";
   str1 += "it under the terms of the GNU General Public License as published by|";
@@ -996,7 +998,21 @@ void Worker::about()
 
 bool Worker::configure()
 {
-  return wconfig->configure();
+  std::string oldsbt;
+  bool erg;
+
+  oldsbt = statebar->getText();
+  setStatebarText( catalog.getLocale( 595 ) );
+  setWaitCursor();
+  erg = wconfig->configure();
+
+  setStatebarText( oldsbt.c_str() );
+  unsetWaitCursor();
+
+  restart = erg;
+  configstarted = true;
+
+  return erg;
 }
 
 Lister *Worker::getOtherLister(Lister *l1)
@@ -1058,9 +1074,20 @@ int Worker::interpret(List* coms,ActionMessage *msg)
   coms->closeEnum(id);
 #else
   WPUContext *wpu;
+  List *parentActionList = NULL;
+
+  if ( ( coms == NULL ) || ( msg == NULL ) ) return 1;
   
   mainwin->forbidUserInput();
   usr1signal = 0;
+
+  if ( ( coms->size() < 1 ) && ( msg->filetype != NULL ) ) {
+    // no action and filetype, so use list with parentaction
+    parentActionList = new List();
+    parentActionList->addElement( new ParentActionOp() );
+    coms = parentActionList;
+  }
+
   wpu = new WPUContext( this, coms, msg );
   while( wpu->next( msg ) == 0 ) {
     if ( usr1signal != 0 ) {
@@ -1073,6 +1100,21 @@ int Worker::interpret(List* coms,ActionMessage *msg)
     }
   }
   delete wpu;
+
+  if ( parentActionList != NULL ) {
+    int id;
+    FunctionProto *fp1;
+
+    id = parentActionList->initEnum();
+    fp1 = (FunctionProto*)parentActionList->getFirstElement( id );
+    while ( fp1 != NULL ) {
+      delete fp1;
+      fp1 = (FunctionProto*)parentActionList->getNextElement( id );
+    }
+    parentActionList->closeEnum( id );
+    delete parentActionList;
+  }
+
   mainwin->permitUserInput();
 #endif
   return 0;
