@@ -1,7 +1,7 @@
 /* worker_commands.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: worker_commands.cc,v 1.25 2004/03/18 07:12:38 ralf Exp $ */
+/* $Id: worker_commands.cc,v 1.27 2004/10/27 21:10:16 ralf Exp $ */
 
 #include "worker.h"
 #include "worker_commands.h"
@@ -163,6 +163,12 @@ FunctionProto *Worker::getCommand4ID(int id)
     case 45:
       fpr = new ShowDirCacheOp();
       break;
+    case 46:
+      fpr = new ParentActionOp();
+      break;
+    case 47:
+      fpr = new NoOperationOp();
+      break;
     default:
       fpr=new DNDAction();
   }
@@ -218,6 +224,8 @@ int Worker::getID4Command(FunctionProto *fpr)
   else if(fpr->isName("ChOwnOp")==true) type=43;
   else if(fpr->isName("ScriptOp")==true) type=44;
   else if ( fpr->isName( ShowDirCacheOp::name ) == true ) type = 45;
+  else if ( fpr->isName( ParentActionOp::name ) == true ) type = 46;
+  else if ( fpr->isName( NoOperationOp::name ) == true ) type = 47;
   return type;
 }
 
