@@ -1,7 +1,7 @@
 /* worker_commands.h
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: worker_commands.h,v 1.24 2004/03/18 07:12:41 ralf Exp $ */
+/* $Id: worker_commands.h,v 1.26 2004/10/27 21:10:18 ralf Exp $ */
 
 #ifndef WORKER_COMMANDS_H
 #define WORKER_COMMANDS_H
@@ -49,6 +49,8 @@
 #include "chownop.h"
 #include "scriptop.h"
 #include "showdircacheop.h"
+#include "parentactionop.h"
+#include "nooperationop.h"
 
 #include "listermode.h"
 #include "normalmode.h"
