@@ -1,7 +1,7 @@
 /* worker_locale.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2001-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: worker_locale.cc,v 1.154 2004/07/04 20:05:25 ralf Exp $ */
+/* $Id: worker_locale.cc,v 1.159 2004/10/31 16:48:33 ralf Exp $ */
 
 #include "worker_locale.h"
 #include "wconfig.h"
 #include "worker.h"
 
-#define CATENTRY 595
-#define CATCOMENTRY 46
+#define CATENTRY 617
+#define CATCOMENTRY 48
 #define CATFLAGENTRY 54
 
 #define min(a,b) (((a)<(b))?(a):(b))
@@ -678,6 +678,36 @@ use <, <=, ==, >=, >, != as usual and ~= for string comparison using regular|exp
 when both sides are strings (\"\" or ${}) they are compared as strings,|otherwise with the \
 numerical value. So ( \"5\" > \"06\" ) is true while ( 5 > \"06\" ) is false!" );
   catalog[594]=dupstring( "Move file %s" );
+  catalog[595]=dupstring( "Configuring Worker..." );
+  catalog[596]=dupstring( "New child type" );
+  catalog[597]=dupstring( "This filetype contains child filetypes!|Really delete this and all child types?" );
+  catalog[598]=dupstring( "Cut" );
+  catalog[599]=dupstring( "Paste as root type" );
+  catalog[600]=dupstring( "Paste as child type" );
+  catalog[601]=dupstring( "Select color mode:" );
+  catalog[602]=dupstring( "Default colors" );
+  catalog[603]=dupstring( "Custom colors" );
+  catalog[604]=dupstring( "Colors from external program" );
+  catalog[605]=dupstring( "Colors from parent type" );
+  catalog[606]=dupstring( "Select custom colors" );
+  catalog[607]=dupstring( "unselected" );
+  catalog[608]=dupstring( "selected" );
+  catalog[609]=dupstring( "unselected, active" );
+  catalog[610]=dupstring( "selected, active" );
+  catalog[611]=dupstring( "External program:" );
+  catalog[612]=dupstring( "Colors" );
+  catalog[613]=dupstring( "Enter the program you want to use for the color selection.|\
+You can use the usual flags {f} and {F} for the filename and the filename|\
+with full path. The program you enter needs to output a single line with|\
+8 integer values which describe the colors. The number is the color number|\
+from the palette configuration starting with 0. The colors describe in this|\
+order: the color for the unselected state (foreground followed by background),|\
+the selected color (fg and bg), the color for the active state (fg and bg) and|\
+the color while active and selected (fg and bg). As an example, the output for|\
+the default colors would be: 1 0 2 1 1 7 2 7" );
+  catalog[614]=dupstring( "Add files of same filetype:" );
+  catalog[615]=dupstring( "Add file" );
+  catalog[616]=dupstring( "Remove from list" );
 
 //  catalog[]=dupstring( "" );
   
@@ -727,6 +757,8 @@ numerical value. So ( \"5\" > \"06\" ) is true while ( 5 > \"06\" ) is false!" )
   catalogcom[43]=dupstring( "change owner" );
   catalogcom[44]=dupstring( "script" );
   catalogcom[45]=dupstring( "show directory cache" );
+  catalogcom[46]=dupstring( "start action of parent type" );
+  catalogcom[47]=dupstring( "do nothing" );
   
   catalogflag[0]=dupstring( "Stringrequester" );
   catalogflag[1]=dupstring( "path" );
