@@ -1,7 +1,7 @@
 /* wpucontext.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
  * Copyright (C) 2002-2004 Ralf Hoffmann.
- * You can contact me at: ralf.hoffmann@epost.de
+ * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: wpucontext.cc,v 1.19 2004/06/13 21:35:33 ralf Exp $ */
+/* $Id: wpucontext.cc,v 1.21 2004/12/12 21:22:24 ralf Exp $ */
 
 #include "wpucontext.h"
 #include "scriptop.h"
@@ -151,7 +151,7 @@ WPUContext::WPUContext( Worker *w, List *coms, ActionMessage *msg )
   if ( msg != NULL ) {
     if ( msg->mode != ActionMessage::AM_MODE_NORMAL ) {
       if ( msg->mode == ActionMessage::AM_MODE_DNDACTION ) {
-        DNDMsg *dndmsg = (DNDMsg*)msg->extraInfo;
+        const DNDMsg *dndmsg = msg->dndmsg;
         ListerMode *lm1;
         NormalMode *nm1;
 
